@@ -89,7 +89,7 @@ namespace PyXRootD
 
     else {
       XrdCl::LocationInfo *response = 0;
-      status = self->filesystem->Locate( path, flags, response, timeout );
+      async( status = self->filesystem->Locate( path, flags, response, timeout ) );
       pyresponse = ConvertType<XrdCl::LocationInfo>( response );
       delete response;
     }
@@ -127,7 +127,7 @@ namespace PyXRootD
 
     else {
       XrdCl::LocationInfo *response = 0;
-      status = self->filesystem->DeepLocate( path, flags, response, timeout );
+      async( status = self->filesystem->DeepLocate( path, flags, response, timeout ) );
       pyresponse = ConvertType<XrdCl::LocationInfo>( response );
       delete response;
     }
@@ -164,7 +164,7 @@ namespace PyXRootD
     }
 
     else {
-      status = self->filesystem->Mv( source, dest, timeout );
+      async( status = self->filesystem->Mv( source, dest, timeout ) );
     }
 
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
@@ -202,7 +202,7 @@ namespace PyXRootD
 
     else {
       XrdCl::Buffer *response = 0;
-      status = self->filesystem->Query( queryCode, argbuffer, response, timeout );
+      async( status = self->filesystem->Query( queryCode, argbuffer, response, timeout ) );
       pyresponse = ConvertType<XrdCl::Buffer>( response );
       delete response;
     }
@@ -238,7 +238,7 @@ namespace PyXRootD
     }
 
     else {
-      status = self->filesystem->Truncate( path, size, timeout );
+      async( status = self->filesystem->Truncate( path, size, timeout ) );
     }
 
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
@@ -270,7 +270,7 @@ namespace PyXRootD
     }
 
     else {
-      status = self->filesystem->Rm( path, timeout );
+      async( status = self->filesystem->Rm( path, timeout ) );
     }
 
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
@@ -305,7 +305,7 @@ namespace PyXRootD
     }
 
     else {
-      status = self->filesystem->MkDir( path, flags, mode, timeout );
+      async( status = self->filesystem->MkDir( path, flags, mode, timeout ) );
     }
 
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
@@ -337,7 +337,7 @@ namespace PyXRootD
     }
 
     else {
-      status = self->filesystem->RmDir( path, timeout );
+      async( status = self->filesystem->RmDir( path, timeout ) );
     }
 
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
@@ -370,7 +370,7 @@ namespace PyXRootD
     }
 
     else {
-      status = self->filesystem->ChMod( path, mode, timeout );
+      async( status = self->filesystem->ChMod( path, mode, timeout ) );
     }
 
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
@@ -401,7 +401,7 @@ namespace PyXRootD
     }
 
     else {
-      status = self->filesystem->Ping( timeout );
+      async( status = self->filesystem->Ping( timeout ) );
     }
 
     pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
@@ -434,7 +434,7 @@ namespace PyXRootD
 
     else {
       XrdCl::StatInfo *response = 0;
-      status = self->filesystem->Stat( path, response, timeout );
+      async( status = self->filesystem->Stat( path, response, timeout ) );
       pyresponse = ConvertType<XrdCl::StatInfo>( response );
       delete response;
     }
@@ -470,7 +470,7 @@ namespace PyXRootD
 
     else {
       XrdCl::StatInfoVFS *response = 0;
-      status = self->filesystem->StatVFS( path, response, timeout );
+      async( status = self->filesystem->StatVFS( path, response, timeout ) );
       pyresponse = ConvertType<XrdCl::StatInfoVFS>( response );
       delete response;
     }
@@ -505,7 +505,7 @@ namespace PyXRootD
 
     else {
       XrdCl::ProtocolInfo *response = 0;
-      status = self->filesystem->Protocol( response, timeout );
+      async( status = self->filesystem->Protocol( response, timeout ) );
       pyresponse = ConvertType<XrdCl::ProtocolInfo>( response );
       delete response;
     }
@@ -543,7 +543,7 @@ namespace PyXRootD
 
     else {
       XrdCl::DirectoryList *list = 0;
-      status = self->filesystem->DirList( path, flags, list, timeout );
+      async( status = self->filesystem->DirList( path, flags, list, timeout ) );
       pyresponse = ConvertType<XrdCl::DirectoryList>( list );
       delete list;
     }
@@ -579,7 +579,7 @@ namespace PyXRootD
 
     else {
       XrdCl::Buffer *response = 0;
-      status = self->filesystem->SendInfo( info, response, timeout );
+      async( status = self->filesystem->SendInfo( info, response, timeout ) );
       pyresponse = ConvertType<XrdCl::Buffer>( response );
       delete response;
     }
@@ -637,8 +637,8 @@ namespace PyXRootD
 
     else {
       XrdCl::Buffer *response = 0;
-      status = self->filesystem->Prepare( files, flags, priority, response,
-                                          timeout );
+      async( status = self->filesystem->Prepare( files, flags, priority, response,
+                                          timeout ) );
       pyresponse = ConvertType<XrdCl::Buffer>( response );
       delete response;
     }
