@@ -5,6 +5,7 @@
 #-------------------------------------------------------------------------------
 
 RCEXP='^[0-9]+\.[0-9]+\.[0-9]+\-rc.*$'
+CERNEXP='^[0-9]+\.[0-9]+\.[0-9]+\-[0-9]+\.CERN.*$'
 
 #-------------------------------------------------------------------------------
 # Find a program
@@ -132,6 +133,14 @@ if test x`echo $VERSION | egrep $RCEXP` != x; then
   VERSION=`echo $VERSION | sed 's/-rc.*//'`
 fi
 
+#-------------------------------------------------------------------------------
+# Deal with CERN releases
+#-------------------------------------------------------------------------------
+if test x`echo $VERSION | egrep $CERNEXP` != x; then
+  RELEASE=`echo $VERSION | sed 's/.*-//'` 
+  VERSION=`echo $VERSION | sed 's/-.*\.CERN//'`
+fi
+
 VERSION=`echo $VERSION | sed 's/-/./g'`
 echo "[i] RPM compliant version: $VERSION-$RELEASE"
 
@@ -219,6 +228,7 @@ cd $CWD
 #-------------------------------------------------------------------------------
 echo "[i] Creating the source RPM..."
 
+# Dirty, dirty hack!
 echo "%_sourcedir $RPMSOURCES" >> $TEMPDIR/rpmmacros
 rpmbuild --define "_topdir $TEMPDIR/rpmbuild"    \
          --define "%_sourcedir $RPMSOURCES"      \
