@@ -60,6 +60,7 @@
 
 #include "XrdOuc/XrdOuca2x.hh"
 #include "XrdOuc/XrdOucEnv.hh"
+#include "XrdOuc/XrdOucLogging.hh"
 #include "XrdOuc/XrdOucSiteName.hh"
 #include "XrdOuc/XrdOucStream.hh"
 #include "XrdOuc/XrdOucUtils.hh"
@@ -215,19 +216,20 @@ int XrdConfig::Configure(int argc, char **argv)
 
    int retc, NoGo = 0, clPort = -1, optbg = 0;
    const char *temp;
-   char c, buff[512], *dfltProt, *libProt = 0, *logfn = 0;
+   char c, buff[512], *dfltProt, *libProt = 0;
    uid_t myUid = 0;
    gid_t myGid = 0;
    extern char *optarg;
    extern int optind, opterr;
+   struct XrdOucLogging::configLogInfo LogInfo;
    int pipeFD[2] = {-1, -1};
    const char *pidFN = 0;
    static const int myMaxc = 80;
    char **urArgv, *myArgv[myMaxc], argBuff[myMaxc*3+8];
    char *argbP = argBuff, *argbE = argbP+sizeof(argBuff)-4;
    char *ifList = 0;
-   int   myArgc = 1, bindArg = 1, urArgc = argc, i;
-   bool noV6, ipV4 = false, ipV6 = false, pureLFN = false;
+   int   myArgc = 1, urArgc = argc, i;
+   bool noV6, ipV4 = false, ipV6 = false;
 
 // Obtain the protocol name we will be using
 //
@@ -303,18 +305,12 @@ int XrdConfig::Configure(int argc, char **argv)
                        Usage(1);
                       }
                  break;
-       case 'k': if (!(bindArg = Log.logger()->ParseKeep(optarg)))
+       case 'k': if (!(LogInfo.keepV = Log.logger()->ParseKeep(optarg)))
                     {Log.Emsg("Config","Invalid -k argument -",optarg);
                      Usage(1);
                     }
                  break;
-       case 'l': if ((pureLFN = *optarg == '=')) optarg++;
-                 if (!*optarg)
-                    {Log.Emsg("Config", "Logfile name not specified.");
-                     Usage(1);
-                    }
-                 if (logfn) free(logfn);
-                 logfn = strdup(optarg);
+       case 'l': LogInfo.logArg = optarg;
                  break;
        case 'L': if (!*optarg)
                     {Log.Emsg("Config", "Protocol library path not specified.");
@@ -343,7 +339,7 @@ int XrdConfig::Configure(int argc, char **argv)
        case 'v': cerr <<XrdVSTRING <<endl;
                  _exit(0);
                  break;
-       case 'z': Log.logger()->setHiRes();
+       case 'z': LogInfo.hiRes = true;
                  break;
 
        default: if (optopt == '-' && *(argv[optind]+1) == '-')
@@ -401,25 +397,22 @@ int XrdConfig::Configure(int argc, char **argv)
    if (optbg)
    {
 #ifdef WIN32
-      XrdOucUtils::Undercover(&Log, !logfn);
+      XrdOucUtils::Undercover(&Log, !LogInfo.logArg);
 #else
       if (pipe( pipeFD ) == -1)
          {Log.Emsg("Config", errno, "create a pipe"); exit(17);}
-      XrdOucUtils::Undercover(Log, !logfn, pipeFD);
+      XrdOucUtils::Undercover(Log, !LogInfo.logArg, pipeFD);
 #endif
    }
 
 // Bind the log file if we have one
 //
-   if (logfn)
-      {char *lP;
-       if (!pureLFN && !(logfn = XrdOucUtils::subLogfn(Log,myInsName,logfn)))
-          _exit(16);
+   if (LogInfo.logArg)
+      {LogInfo.xrdEnv = &theEnv;
+       LogInfo.iName  = myInsName;
+       LogInfo.cfgFn  = ConfigFN;
+       if (!XrdOucLogging::configLog(Log, LogInfo)) _exit(16);
        Log.logger()->AddMsg(XrdBANNER);
-       if (Log.logger()->Bind(logfn, bindArg)) exit(19);
-       if ((lP = rindex(logfn,'/'))) {*(lP+1) = '\0'; lP = logfn;}
-          else lP = (char *)"./";
-       XrdOucEnv::Export("XRDLOGDIR", lP);
       }
 
 // Get the full host name. In theory, we should always get some kind of name.
@@ -545,12 +538,11 @@ int XrdConfig::Configure(int argc, char **argv)
    temp = (NoGo ? " initialization failed." : " initialization completed.");
    sprintf(buff, "%s:%d", myInstance, PortTCP);
    Log.Say("------ ", buff, temp);
-   if (logfn)
+   if (LogInfo.logArg)
       {strcat(buff, " running ");
        retc = strlen(buff);
        XrdSysUtils::FmtUname(buff+retc, sizeof(buff)-retc);
        Log.logger()->AddMsg(buff);
-       free(logfn);
       }
    return NoGo;
 }
@@ -855,8 +847,8 @@ int XrdConfig::setFDL()
 
 // Set the limit to the maximum allowed
 //
-   if (rlim.rlim_max == RLIM_INFINITY) rlim.rlim_max = maxFD;
-   rlim.rlim_cur = rlim.rlim_max;
+   if (rlim.rlim_max == RLIM_INFINITY) rlim.rlim_cur = maxFD;
+      else rlim.rlim_cur = rlim.rlim_max;
 #if (defined(__APPLE__) && defined(MAC_OS_X_VERSION_10_5))
    if (rlim.rlim_cur > OPEN_MAX) rlim.rlim_max = rlim.rlim_cur = OPEN_MAX;
 #endif
@@ -1220,6 +1212,7 @@ int XrdConfig::xbuf(XrdSysError *eDest, XrdOucStream &Config)
    Purpose:  To parse directive: network [wan] [[no]keepalive] [buffsz <blen>]
                                          [kaparms parms] [cache <ct>] [[no]dnr]
                                          [routes <rtype> [use <ifn1>,<ifn2>]]
+                                         [[no]rpipa]
 
              <rtype>: split | common | local
 
@@ -1230,6 +1223,7 @@ int XrdConfig::xbuf(XrdSysError *eDest, XrdOucStream &Config)
              <ct>      Seconds to cache address to name resolutions.
              [no]dnr   do [not] perform a reverse DNS lookup if not needed.
              routes    specifies the network configuration (see reference)
+             [no]rpipa do [not] resolve private IP addresses.
 
    Output: 0 upon success or !0 upon failure.
 */
@@ -1238,6 +1232,7 @@ int XrdConfig::xnet(XrdSysError *eDest, XrdOucStream &Config)
 {
     char *val;
     int  i, n, V_keep = -1, V_nodnr = 0, V_iswan = 0, V_blen = -1, V_ct = -1;
+    int  v_rpip = -1;
     long long llp;
     struct netopts {const char *opname; int hasarg; int opval;
                            int *oploc;  const char *etxt;}
@@ -1251,6 +1246,8 @@ int XrdConfig::xnet(XrdSysError *eDest, XrdOucStream &Config)
         {"dnr",        0, 0, &V_nodnr,  "option"},
         {"nodnr",      0, 1, &V_nodnr,  "option"},
         {"routes",     3, 1, 0,         "routes"},
+        {"rpipa",      0, 1, &v_rpip,   "rpipa"},
+        {"norpipa",    0, 0, &v_rpip,   "norpipa"},
         {"wan",        0, 1, &V_iswan,  "option"}
        };
     int numopts = sizeof(ntopts)/sizeof(struct netopts);
@@ -1322,6 +1319,7 @@ int XrdConfig::xnet(XrdSysError *eDest, XrdOucStream &Config)
         }
 
      if (V_ct >= 0) XrdNetAddr::SetCache(V_ct);
+     if (v_rpip >= 0) XrdInet::netIF.SetRPIPA(v_rpip != 0);
      return 0;
 }
 
