@@ -142,7 +142,7 @@ int XrdAccAuthFile::Close()
 /*                                 g e t P P                                  */
 /******************************************************************************/
   
-int XrdAccAuthFile::getPP(char **path, char **priv)
+int XrdAccAuthFile::getPP(char **path, char **priv, bool &istmplt)
 {
 // char *pp, *bp;
    char *pp;
@@ -158,15 +158,26 @@ int XrdAccAuthFile::getPP(char **path, char **priv)
        return 0;
       }
 
+// Check of objectid specification
+//
+   istmplt = false;
+   *path   = path_buff;
+   if (*pp == '\\')
+      {if (*(pp+1)) pp++;
+          else {Eroute->Emsg("AuthFile", "Object ID missing after '\\'");
+                *path = 0;
+                flags = (DBflags)(flags | dbError);
+               }
+      } else if (*pp != '/') istmplt = true;
+
 // Copy the value since the stream buffer might get overlaid.
 //
 // bp = Copy(path_buff, pp, sizeof(path_buff)-1);
-        Copy(path_buff, pp, sizeof(path_buff)-1);
-   *path = path_buff;
+   if (path) Copy(path_buff, pp, sizeof(path_buff)-1);
 
 // Check if this is really a path or a template
 //
-   if (*path_buff != '/') {*priv = (char *)0; return 1;}
+   if (istmplt) {*priv = (char *)0; return 1;}
 
 // Verify that the path ends correctly (normally we would force a slash to
 // appear at the end but that prevents caps on files. So, we commented the
