@@ -39,7 +39,9 @@ namespace XrdCl
     pStream( 0 ),
     pSocket( 0 ),
     pIncoming( 0 ),
+    pHSIncoming( 0 ),
     pOutgoing( 0 ),
+    pHSOutgoing( 0 ),
     pHandShakeData( 0 ),
     pHandShakeDone( false ),
     pConnectionStarted( 0 ),
@@ -334,7 +336,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     if( pHandShakeData->out )
     {
-      pOutgoing = pHandShakeData->out;
+      pHSOutgoing = pHandShakeData->out;
       pHandShakeData->out = 0;
     }
 
@@ -377,7 +379,7 @@ namespace XrdCl
     Status st;
     if( !pOutMsgDone )
     {
-      if( !(st = WriteCurrentMessage()).IsOK() )
+      if( !(st = WriteCurrentMessage( pOutgoing )).IsOK() )
       {
         OnFault( st );
         return;
@@ -423,6 +425,11 @@ namespace XrdCl
 
     pStream->OnMessageSent( pSubStreamNum, pOutgoing, pOutMsgSize );
     pOutgoing = 0;
+
+    //--------------------------------------------------------------------------
+    // Disable the respective substream if empty
+    //--------------------------------------------------------------------------
+    pStream->DisableIfEmpty( pSubStreamNum );
   }
 
   //----------------------------------------------------------------------------
@@ -431,14 +438,14 @@ namespace XrdCl
   void AsyncSocketHandler::OnWriteWhileHandshaking()
   {
     Status st;
-    if( !pOutgoing )
+    if( !pHSOutgoing )
     {
       if( !(st = DisableUplink()).IsOK() )
         OnFaultWhileHandshaking( st );
       return;
     }
 
-    if( !(st = WriteCurrentMessage()).IsOK() )
+    if( !(st = WriteCurrentMessage( pHSOutgoing )).IsOK() )
     {
       OnFaultWhileHandshaking( st );
       return;
@@ -446,8 +453,8 @@ namespace XrdCl
 
     if( st.code != suRetry )
     {
-      delete pOutgoing;
-      pOutgoing = 0;
+      delete pHSOutgoing;
+      pHSOutgoing = 0;
       if( !(st = DisableUplink()).IsOK() )
         OnFaultWhileHandshaking( st );
       return;
@@ -457,14 +464,14 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Write the current message
   //----------------------------------------------------------------------------
-  Status AsyncSocketHandler::WriteCurrentMessage()
+  Status AsyncSocketHandler::WriteCurrentMessage( Message *toWrite )
   {
     Log *log = DefaultEnv::GetLog();
 
     //--------------------------------------------------------------------------
     // Try to write down the current message
     //--------------------------------------------------------------------------
-    Message  *msg             = pOutgoing;
+    Message  *msg             = toWrite;
     uint32_t  leftToBeWritten = msg->GetSize()-msg->GetCursor();
 
     while( leftToBeWritten )
@@ -482,7 +489,7 @@ namespace XrdCl
         //----------------------------------------------------------------------
         // Actual socket error error!
         //----------------------------------------------------------------------
-        pOutgoing->SetCursor( 0 );
+        toWrite->SetCursor( 0 );
         return Status( stError, errSocketError, errno );
       }
       msg->AdvanceCursor( status );
@@ -493,8 +500,8 @@ namespace XrdCl
     // We have written the message successfully
     //--------------------------------------------------------------------------
     log->Dump( AsyncSockMsg, "[%s] Wrote a message: %s (0x%x), %d bytes",
-               pStreamName.c_str(), pOutgoing->GetDescription().c_str(),
-               pOutgoing, pOutgoing->GetSize() );
+               pStreamName.c_str(), toWrite->GetDescription().c_str(),
+               toWrite, toWrite->GetSize() );
     return Status();
   }
 
@@ -602,7 +609,7 @@ namespace XrdCl
     // Read the message and let the transport handler look at it when
     // reading has finished
     //--------------------------------------------------------------------------
-    Status st = ReadMessage();
+    Status st = ReadMessage( pHSIncoming );
     if( !st.IsOK() )
     {
       OnFaultWhileHandshaking( st );
@@ -615,8 +622,8 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // OK, we have a new message, let's deal with it;
     //--------------------------------------------------------------------------
-    pHandShakeData->in = pIncoming;
-    pIncoming = 0;
+    pHandShakeData->in = pHSIncoming;
+    pHSIncoming = 0;
     st = pTransport->HandShake( pHandShakeData, *pChannelData );
     ++pHandShakeData->step;
     delete pHandShakeData->in;
@@ -633,7 +640,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     if( pHandShakeData->out )
     {
-      pOutgoing = pHandShakeData->out;
+      pHSOutgoing = pHandShakeData->out;
       pHandShakeData->out = 0;
       Status st;
       if( !(st = EnableUplink()).IsOK() )
@@ -668,35 +675,35 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Read a message
   //----------------------------------------------------------------------------
-  Status AsyncSocketHandler::ReadMessage()
+  Status AsyncSocketHandler::ReadMessage( Message *&toRead )
   {
-    if( !pIncoming )
+    if( !toRead )
     {
       pHeaderDone = false;
-      pIncoming   = new Message();
+      toRead      = new Message();
     }
 
     Status  st;
     Log    *log = DefaultEnv::GetLog();
     if( !pHeaderDone )
     {
-      st = pTransport->GetHeader( pIncoming, pSocket->GetFD() );
+      st = pTransport->GetHeader( toRead, pSocket->GetFD() );
       if( st.IsOK() && st.code == suDone )
       {
         log->Dump( AsyncSockMsg,
                   "[%s] Received message header, size: %d",
-                  pStreamName.c_str(), pIncoming->GetCursor() );
+                  pStreamName.c_str(), toRead->GetCursor() );
         pHeaderDone = true;
       }
       else
         return st;
     }
 
-    st = pTransport->GetBody( pIncoming, pSocket->GetFD() );
+    st = pTransport->GetBody( toRead, pSocket->GetFD() );
     if( st.IsOK() && st.code == suDone )
     {
       log->Dump( AsyncSockMsg, "[%s] Received a message of %d bytes",
-                 pStreamName.c_str(), pIncoming->GetSize() );
+                 pStreamName.c_str(), toRead->GetSize() );
     }
     return st;
   }
@@ -728,10 +735,10 @@ namespace XrdCl
     Log *log = DefaultEnv::GetLog();
     log->Error( AsyncSockMsg, "[%s] Socket error while handshaking: %s",
                 pStreamName.c_str(), st.ToString().c_str() );
-    delete pIncoming;
-    delete pOutgoing;
-    pIncoming = 0;
-    pOutgoing = 0;
+    delete pHSIncoming;
+    delete pHSOutgoing;
+    pHSIncoming = 0;
+    pHSOutgoing = 0;
 
     pStream->OnConnectError( pSubStreamNum, st );
   }
