@@ -29,6 +29,7 @@
 #include "XrdCl/XrdClConstants.hh"
 #include "XrdCl/XrdClLog.hh"
 #include "XrdCl/XrdClUglyHacks.hh"
+#include "XrdCl/XrdClRedirectorRegistry.hh"
 
 #include <ctime>
 
@@ -103,6 +104,17 @@ namespace
         return pMsg;
       }
 
+      //------------------------------------------------------------------------
+      // Get underlying message filter sid
+      //------------------------------------------------------------------------
+      uint16_t GetSid() const
+      {
+	if (pFilter)
+	  return pFilter->GetSid();
+
+	return 0;
+      }
+
     private:
       FilterHandler(const FilterHandler &other);
       FilterHandler &operator = (const FilterHandler &other);
@@ -279,10 +291,20 @@ namespace XrdCl
   Status Channel::Send( Message              *msg,
                         OutgoingMsgHandler   *handler,
                         bool                  stateful,
-                        time_t                expires )
+                        time_t                expires,
+                        VirtualRedirector    *redirector )
 
   {
     PathID path = pTransport->Multiplex( msg, pChannelData );
+
+    if( redirector )
+    {
+      XRootDStatus st = redirector->HandleRequest( msg, pStreams[path.down] );
+      if( st.IsOK() )
+        handler->OnStatusReady( msg, Status() );
+      return st;
+    }
+
     return pStreams[path.up]->Send( msg, handler, stateful, expires );
   }
 
