@@ -210,19 +210,6 @@ namespace XrdCl
       return true;
     }
 
-    while( !pPollerPool.empty() )
-    {
-      XrdSys::IOEvents::Poller *poller = pPollerPool.back();
-      pPollerPool.pop_back();
-
-      if( !poller ) continue;
-
-      scopedLock.UnLock();
-      poller->Stop();
-      delete poller;
-      scopedLock.Lock( &pMutex );
-    }
-
     SocketMap::iterator  it;
     const char          *errMsg = 0;
 
@@ -240,6 +227,21 @@ namespace XrdCl
       helper->channel = 0;
     }
 
+    while( !pPollerPool.empty() )
+    {
+      XrdSys::IOEvents::Poller *poller = pPollerPool.back();
+      pPollerPool.pop_back();
+
+      if( !poller ) continue;
+
+      scopedLock.UnLock();
+      poller->Stop();
+      delete poller;
+      scopedLock.Lock( &pMutex );
+    }
+    pNext == pPollerPool.end();
+    pPollerMap.clear();
+
     return true;
   }
 
@@ -547,7 +549,7 @@ namespace XrdCl
     if( itr == pPollerMap.end())
     {
       XrdSys::IOEvents::Poller* poller = GetNextPoller();
-      pPollerMap[socket->GetChannelID()] = std::make_pair(poller, 1);
+      pPollerMap[socket->GetChannelID()] = std::make_pair(poller, (size_t)1);
       return poller;
     }
 
