@@ -33,6 +33,8 @@
 #include "XrdCl/XrdClOutQueue.hh"
 #include "XrdCl/XrdClMonitor.hh"
 #include "XrdCl/XrdClAsyncSocketHandler.hh"
+#include "XrdCl/XrdClMessageUtils.hh"
+#include "XrdCl/XrdClXRootDTransport.hh"
 
 #include <sys/types.h>
 #include <algorithm>
@@ -323,6 +325,27 @@ namespace XrdCl
     return st;
   }
 
+
+  //------------------------------------------------------------------------
+  // Queue a virtual response
+  //------------------------------------------------------------------------
+  Status Stream::ReceiveVirtual( Message *msg )
+  {
+    //--------------------------------------------------------------------------
+    // Check the session ID and bounce if needed
+    //--------------------------------------------------------------------------
+    if( msg->GetSessionId() &&
+        (pSubStreams[0]->status != Socket::Connected ||
+        pSessionId != msg->GetSessionId()) )
+      return Status( stError, errInvalidSession );
+
+    Log *log = DefaultEnv::GetLog();
+    log->Dump( PostMasterMsg, "[%s] Queuing virtual response: 0x%x.", pStreamName.c_str(), msg );
+    pJobManager->QueueJob( pQueueIncMsgJob, msg );
+
+    return Status();
+  }
+
   //----------------------------------------------------------------------------
   // Force connection
   //----------------------------------------------------------------------------
@@ -406,6 +429,26 @@ namespace
 
 namespace XrdCl
 {
+  Status Stream::RequestClose( Message *response )
+  {
+    ServerResponse *rsp = reinterpret_cast<ServerResponse*>( response->GetBuffer() );
+    if( rsp->hdr.dlen < 4 ) return Status( stError );
+    Message            *msg;
+    ClientCloseRequest *req;
+    MessageUtils::CreateRequest( msg, req );
+    req->requestid = kXR_close;
+    memcpy( req->fhandle, reinterpret_cast<uint8_t*>( rsp->body.buffer.data ), 4 );
+    XRootDTransport::SetDescription( msg );
+    msg->SetSessionId( pSessionId );
+    NullResponseHandler *handler = new NullResponseHandler();
+    MessageSendParams params;
+    params.timeout         = 0;
+    params.followRedirects = false;
+    params.stateful        = true;
+    MessageUtils::ProcessSendParams( params );
+    return MessageUtils::SendMessage( *pUrl, msg, handler, params );
+  }
+
   //----------------------------------------------------------------------------
   // Call back when a message has been reconstructed
   //----------------------------------------------------------------------------
@@ -422,6 +465,13 @@ namespace XrdCl
     if( streamAction & TransportHandler::DigestMsg )
       return;
 
+    if( streamAction & TransportHandler::RequestClose )
+    {
+      RequestClose( msg );
+      delete msg;
+      return;
+    }
+
     //--------------------------------------------------------------------------
     // No handler, we cache and see what comes later
     //--------------------------------------------------------------------------
@@ -488,6 +538,19 @@ namespace XrdCl
     return std::make_pair( h.msg, h.handler );
   }
 
+  void Stream::DisableIfEmpty( uint16_t subStream )
+  {
+    XrdSysMutexHelper scopedLock( pMutex );
+    Log *log = DefaultEnv::GetLog();
+
+    if( pSubStreams[subStream]->outQueue->IsEmpty() )
+    {
+      log->Dump( PostMasterMsg, "[%s] All messages consumed, disable uplink",
+                 pSubStreams[subStream]->socket->GetStreamName().c_str() );
+      pSubStreams[subStream]->socket->DisableUplink();
+    }
+  }
+
   //----------------------------------------------------------------------------
   // Call when a message is written to the socket
   //----------------------------------------------------------------------------
