@@ -68,6 +68,8 @@
 #include "XrdFfs/XrdFfsFsinfo.hh"
 #include "XrdPosix/XrdPosixXrootd.hh"
 
+#define MAXROOTURLLEN 1024 // this is also defined in other files
+
 struct XROOTDFS {
     char *rdr;
     char *cns;
@@ -77,21 +79,16 @@ struct XROOTDFS {
     char *urlcachelife;
     bool ofsfwd;
     int  nworkers;
+    int  maxfd;
 };
 
 int cwdfd; // File descript of the initial working dir
 
 struct XROOTDFS xrootdfs;
-static struct fuse_opt xrootdfs_opts[13];
+static struct fuse_opt xrootdfs_opts[14];
 
 enum { OPT_KEY_HELP, OPT_KEY_SECSSS, };
 
-/*
-char *rdr, *cns, *fastls="", *daemon_user;
-//enum Boolean {false, true} ofsfwd;
-bool ofsfwd;
-*/
-
 static void* xrootdfs_init(struct fuse_conn_info *conn)
 {
     struct passwd pw, *pwp;
@@ -119,12 +116,18 @@ static void* xrootdfs_init(struct fuse_conn_info *conn)
             }
         }
         if (i == len) getpwuid_r(atoi(xrootdfs.daemon_user), &pw, pwbuf, pwbuflen, &pwp);
-        setgid((gid_t)pw.pw_gid);
-        setuid((uid_t)pw.pw_uid);
+        if( setgid((gid_t)pw.pw_gid) != 0 )
+            syslog( LOG_ERR, "ERROR: Unable to set gid to %d", pw.pw_gid );
+        if( setuid((uid_t)pw.pw_uid) != 0 )
+            syslog( LOG_ERR, "ERROR: Unable to set uid to %d", pw.pw_uid );
         prctl(PR_SET_DUMPABLE, 1); // enable core dump after setuid/setgid
     }
     free(pwbuf);
 
+/* put Xrootd related initialization calls here, after fuse daemonize itself. */
+    XrdPosixXrootd *abc = new XrdPosixXrootd(-xrootdfs.maxfd);
+    XrdFfsMisc_xrd_init(xrootdfs.rdr,xrootdfs.urlcachelife,0);
+    XrdFfsWcache_init(abc->fdOrigin(), xrootdfs.maxfd);
 /*
    From FAQ:
       Miscellaneous threads should be started from the init() method.
@@ -150,17 +153,17 @@ static int xrootdfs_getattr(const char *path, struct stat *stbuf)
 {
 //  int res, fd;
     int res;
-    char rootpath[1024];
-    uid_t user_uid, uid;
-    gid_t user_gid, gid;
+    char rootpath[MAXROOTURLLEN];
+//    uid_t user_uid, uid;
+//    gid_t user_gid, gid;
 
-    user_uid = fuse_get_context()->uid;
-    uid = getuid();
+//    user_uid = fuse_get_context()->uid;
+//    uid = getuid();
 
-    user_gid = fuse_get_context()->gid;
-    gid = getgid();
+//    user_gid = fuse_get_context()->gid;
+//    gid = getgid();
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
 
     rootpath[0]='\0';
 /*
@@ -178,9 +181,9 @@ static int xrootdfs_getattr(const char *path, struct stat *stbuf)
 
     if (xrootdfs.cns != NULL && xrootdfs.fastls != NULL)
     {
-        strcat(rootpath,xrootdfs.cns);
-        strcat(rootpath,path);
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         res = XrdFfsPosix_stat(rootpath, stbuf);
     }
     else
@@ -211,9 +214,9 @@ static int xrootdfs_getattr(const char *path, struct stat *stbuf)
             if (xrootdfs.cns != NULL && xrootdfs.fastls != NULL && strcmp(xrootdfs.fastls,"RDR") == 0)
             {
                 rootpath[0]='\0';
-                strcat(rootpath,xrootdfs.rdr);
-                strcat(rootpath,path);
-                XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+                strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+                strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
+                XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
                 XrdFfsPosix_stat(rootpath, stbuf);
 //                stbuf->st_uid = user_uid;
 //                stbuf->st_gid = user_gid;
@@ -239,9 +242,9 @@ static int xrootdfs_getattr(const char *path, struct stat *stbuf)
     else
     {
         rootpath[0]='\0';
-        strcat(rootpath,xrootdfs.cns);
-        strcat(rootpath,path);
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         res = XrdFfsPosix_stat(rootpath, stbuf);
 //        stbuf->st_uid = user_uid;
 //        stbuf->st_gid = user_gid;
@@ -297,9 +300,9 @@ static int xrootdfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     (void) offset;
     (void) fi;
 
-    char rootpath[1024];
+    char rootpath[MAXROOTURLLEN];
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
 /* 
    if CNS server is not defined, there is no way to list files in a directory
    because we don't know the data nodes
@@ -307,10 +310,10 @@ static int xrootdfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     if (xrootdfs.cns != NULL)
     {
         rootpath[0]='\0';
-        strcat(rootpath,xrootdfs.cns);
-        strcat(rootpath,path);
+        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         dp = XrdFfsPosix_opendir(rootpath);
         if (dp == NULL)
             return -errno;
@@ -332,7 +335,7 @@ static int xrootdfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     else  /* if there is no CNS, try collect dirents from all known data servers. */
     {
          int i, n;
-         char **dnarray;
+         char **dnarray = NULL;
 
          n = XrdFfsPosix_readdirall(xrootdfs.rdr, path, &dnarray, fuse_get_context()->uid);
 
@@ -357,16 +360,16 @@ static int xrootdfs_mknod(const char *path, mode_t mode, dev_t rdev)
 
     /* On Linux this could just be 'mknod(path, mode, rdev)' but this
        is more portable */
-    char rootpath[1024];
+    char rootpath[MAXROOTURLLEN];
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
     if (S_ISREG(mode))
     {
         rootpath[0]='\0';
-        strcat(rootpath,xrootdfs.rdr);
-        strcat(rootpath,path);
+        strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
 /* 
    Around May 2008, the O_EXCL was added to the _open(). No reason was given. It is removed again 
    due to the following reason (the situation that redirector thinks a file exist while it doesn't):
@@ -390,10 +393,10 @@ static int xrootdfs_mknod(const char *path, mode_t mode, dev_t rdev)
             return 0;
 
         rootpath[0]='\0';
-        strcat(rootpath,xrootdfs.cns);
-        strcat(rootpath,path);
+        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         res = XrdFfsPosix_open(rootpath, O_CREAT | O_EXCL, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH); 
         XrdFfsPosix_close(res);
 /* We actually don't need to care about error by this XrdFfsPosix_open() */
@@ -415,14 +418,14 @@ static int xrootdfs_mkdir(const char *path, mode_t mode)
     rootpath[0]='\0';
 
     if (xrootdfs.cns != NULL)
-        strcat(rootpath,xrootdfs.cns);
+        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
     else
-        strcat(rootpath,xrootdfs.rdr);
+        strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
 
-    strcat(rootpath,path);
+    strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
-    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
+    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
 
     res = XrdFfsPosix_mkdir(rootpath, mode);
     if (res == 0) return 0;
@@ -442,16 +445,16 @@ static int xrootdfs_mkdir(const char *path, mode_t mode)
 static int xrootdfs_unlink(const char *path)
 {
     int res;
-    char rootpath[1024];
+    char rootpath[MAXROOTURLLEN];
 
     rootpath[0]='\0';
-    strcat(rootpath,xrootdfs.rdr);
-    strcat(rootpath,path);
+    strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+    strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
     if (xrootdfs.ofsfwd == true)
     {
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         res = XrdFfsPosix_unlink(rootpath);
     }
     else
@@ -463,10 +466,10 @@ static int xrootdfs_unlink(const char *path)
     if (xrootdfs.cns != NULL && xrootdfs.ofsfwd == false)
     {
         rootpath[0]='\0';
-        strcat(rootpath,xrootdfs.cns);
-        strcat(rootpath,path);
+        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         res = XrdFfsPosix_unlink(rootpath);
         if (res == -1)
             return -errno;
@@ -478,16 +481,16 @@ static int xrootdfs_rmdir(const char *path)
 {
     int res;
 //  struct stat stbuf;
-    char rootpath[1024];
+    char rootpath[MAXROOTURLLEN];
 
     rootpath[0]='\0';
-    strcat(rootpath,xrootdfs.rdr);
-    strcat(rootpath,path);
+    strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+    strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
     if (xrootdfs.ofsfwd == true)
     { 
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         res = XrdFfsPosix_rmdir(rootpath);
     }
     else
@@ -499,10 +502,10 @@ static int xrootdfs_rmdir(const char *path)
     if (xrootdfs.cns != NULL && xrootdfs.ofsfwd == false)
     {
         rootpath[0]='\0';
-        strcat(rootpath,xrootdfs.cns);
-        strcat(rootpath,path);
+        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         res = XrdFfsPosix_rmdir(rootpath);
         if (res == -1)
             return -errno;
@@ -537,16 +540,16 @@ static int xrootdfs_symlink(const char *from, const char *to)
 static int xrootdfs_rename(const char *from, const char *to)
 {
     int res;
-    char from_path[1024], to_path[1024];
+    char from_path[MAXROOTURLLEN], to_path[MAXROOTURLLEN];
     struct stat stbuf;
 
     from_path[0]='\0';
-    strcat(from_path, xrootdfs.rdr);
-    strcat(from_path, from);
+    strncat(from_path, xrootdfs.rdr, MAXROOTURLLEN - strlen(from_path) -1);
+    strncat(from_path, from, MAXROOTURLLEN - strlen(from_path) -1);
 
     to_path[0]='\0';
-    strcat(to_path, xrootdfs.rdr);
-    strcat(to_path, to);
+    strncat(to_path, xrootdfs.rdr, MAXROOTURLLEN - strlen(to_path) -1);
+    strncat(to_path, to, MAXROOTURLLEN - strlen(to_path) -1);
 /*
   1. do actual renaming on data servers if if is a file in order to speed up
      renaming
@@ -554,8 +557,8 @@ static int xrootdfs_rename(const char *from, const char *to)
      are renamed individually (in order for the .pfn pointing back correctly).
  */
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
-    XrdFfsMisc_xrd_secsss_editurl(from_path, fuse_get_context()->uid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
+    XrdFfsMisc_xrd_secsss_editurl(from_path, fuse_get_context()->uid, 0);
 
     XrdFfsPosix_stat(from_path, &stbuf);
     if (S_ISDIR(stbuf.st_mode)) /* && xrootdfs.cns == NULL && xrootdfs.ofsfwd == false) */
@@ -575,12 +578,12 @@ static int xrootdfs_rename(const char *from, const char *to)
     if (xrootdfs.cns != NULL && xrootdfs.ofsfwd == false)
     {
         from_path[0]='\0';
-        strcat(from_path, xrootdfs.cns);
-        strcat(from_path, from);
+        strncat(from_path, xrootdfs.cns, MAXROOTURLLEN - strlen(from_path) -1);
+        strncat(from_path, from, MAXROOTURLLEN - strlen(from_path) -1);
 
         to_path[0]='\0';
-        strcat(to_path, xrootdfs.cns);
-        strcat(to_path, to);
+        strncat(to_path, xrootdfs.cns, MAXROOTURLLEN - strlen(to_path) -1);
+        strncat(to_path, to, MAXROOTURLLEN - strlen(to_path) -1);
 
         res = XrdFfsPosix_rename(from_path, to_path);
         if (res == -1)
@@ -661,16 +664,16 @@ static int xrootdfs_ftruncate(const char *path, off_t size,
 static int xrootdfs_truncate(const char *path, off_t size)
 {
     int res;
-    char rootpath[1024];
+    char rootpath[MAXROOTURLLEN];
 
     rootpath[0]='\0';
-    strcat(rootpath,xrootdfs.rdr);
-    strcat(rootpath,path);
+    strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+    strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
     if (xrootdfs.ofsfwd == true)
     {
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         res = XrdFfsPosix_truncate(rootpath, size);
     }
     else
@@ -682,10 +685,10 @@ static int xrootdfs_truncate(const char *path, off_t size)
     if (xrootdfs.cns != NULL && xrootdfs.ofsfwd == false)
     {
         rootpath[0]='\0';
-        strcat(rootpath,xrootdfs.cns);
-        strcat(rootpath,path);
+        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
         res = XrdFfsPosix_truncate(rootpath, size);
         if (res == -1)
             return -errno;
@@ -713,13 +716,13 @@ static int xrootdfs_utimens(const char *path, const struct timespec ts[2])
 
 static int xrootdfs_open(const char *path, struct fuse_file_info *fi)
 {
-    int res;
-    char rootpath[1024]="";
-    strcat(rootpath,xrootdfs.rdr);
-    strcat(rootpath,path);
+    int res, lid = 1;
+    char rootpath[MAXROOTURLLEN]="";
+    strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+    strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
-    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, &lid);
+    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, &lid);
     res = XrdFfsPosix_open(rootpath, fi->flags, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
     if (res == -1)
         return -errno;
@@ -772,7 +775,7 @@ static int xrootdfs_statfs(const char *path, struct statvfs *stbuf)
 //  long long size;
 
 //    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
-#ifndef __macos__
+#ifndef __APPLE__
     stbuf->f_bsize = 1024;
 #else
     stbuf->f_bsize = 1024 * 128; // work around 32 bit fsblkcnt_t in struct statvfs on Mac OSX
@@ -837,7 +840,7 @@ static int xrootdfs_release(const char *path, struct fuse_file_info *fi)
 
     int fd, oflag;
     struct stat xrdfile, cnsfile;
-    char rootpath[1024];
+    char rootpath[MAXROOTURLLEN];
 
     fd = (int) fi->fh;
     XrdFfsWcache_flush(fd);
@@ -859,13 +862,13 @@ static int xrootdfs_release(const char *path, struct fuse_file_info *fi)
     char *lasts_xattr[256], *lasts_tokens[128];
 
     rootpath[0]='\0';
-    strcat(rootpath,xrootdfs.rdr);
-    strcat(rootpath,path);
+    strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+    strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 /*
  * Get xrootd token info from data nodes. And set the token info on CNS
  */
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
-    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
+    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
     xrdtoken[0]='\0';
     res = XrdFfsPosix_getxattr(rootpath, "xroot.xattr", xattr, 256);
     if (res != -1)
@@ -892,14 +895,14 @@ static int xrootdfs_release(const char *path, struct fuse_file_info *fi)
     }
 
     rootpath[0]='\0';
-    strcat(rootpath,xrootdfs.cns);
-    strcat(rootpath,path);
+    strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
+    strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
     if (xrdtoken[0] != '\0' && strstr(path,"?oss.cgroup=") == NULL)
     {
-        strcat(rootpath,"?oss.cgroup=");
-        strcat(rootpath,xrdtoken);
+        strncat(rootpath,"?oss.cgroup=", MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,xrdtoken, MAXROOTURLLEN - strlen(rootpath) -1);
     }
 
     if (XrdFfsPosix_stat(rootpath,&cnsfile) == -1)
@@ -928,12 +931,11 @@ static int xrootdfs_release(const char *path, struct fuse_file_info *fi)
 static int xrootdfs_fsync(const char *path, int isdatasync,
                      struct fuse_file_info *fi)
 {
-    /* Just a stub.  This method is optional and can safely be left
-       unimplemented */
+    int fd;
 
-    (void) path;
-    (void) isdatasync;
-    (void) fi;
+    fd = (int) fi->fh;
+    XrdFfsWcache_flush(fd);
+    XrdFfsPosix_fsync(fd);
     return 0;
 }
 
@@ -975,21 +977,21 @@ static int xrootdfs_getxattr(const char *path, const char *name, char *value,
                     size_t size)
 {
     int xattrlen;
-    char rootpath[1024]="";
-    char rooturl[1024]="";
+    char rootpath[MAXROOTURLLEN]="";
+    char rooturl[MAXROOTURLLEN]="";
 
     if (!strcmp(name,"xroot.url"))
     {
         errno = 0;
-        strcat(rootpath,xrootdfs.rdr);
-        strcat(rootpath,path);
+        strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
 //        XrdFfsMisc_get_current_url(rootpath, rooturl);
-        strcat(rooturl, rootpath);
+        strcpy(rooturl, rootpath);
 
         if (size == 0)
             return strlen(rooturl);
-        else if (size >= strlen(rooturl)) 
+        else if (size > strlen(rooturl)) // check the size to make sure strcat(value, rooturl) is safe
         {
             size = strlen(rooturl);
             if (size != 0) 
@@ -1018,7 +1020,7 @@ static int xrootdfs_getxattr(const char *path, const char *name, char *value,
             free(hostlist);
             return xattrlen;
         }
-        else if (size >= strlen(hostlist))
+        else if (size > strlen(hostlist))
         {
             size = strlen(hostlist);
             if (size != 0)
@@ -1045,7 +1047,7 @@ static int xrootdfs_getxattr(const char *path, const char *name, char *value,
 
         if (size == 0)
             return strlen(nworkers);
-        else if (size >= strlen(nworkers))
+        else if (size > strlen(nworkers))
         {
             size = strlen(nworkers);
             if (size != 0)
@@ -1063,33 +1065,38 @@ static int xrootdfs_getxattr(const char *path, const char *name, char *value,
     }
     else if (!strcmp(name, "xrootdfs.file.permission"))
     {
-        char xattr[256];
-        strcat(rootpath,xrootdfs.rdr);
-        strcat(rootpath,path);
+        char xattr[256]="";
+        strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-        XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+        XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
+        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
 
         xattrlen = XrdFfsPosix_getxattr(rootpath, "xroot.xattr.ofs.ap", xattr, 255);
         if (size == 0) 
             return xattrlen;
-        else
+        else if (size > (size_t)xattrlen)
+        {
+            strncpy(value, xattr, size);
             size = xattrlen;
-
-        strncpy(value, xattr, size);
-        value[size] = '\0';
-       
-        return size;
+            value[size] = '\0';
+            return size;
+        }
+        else
+        {
+            errno = ERANGE;
+            return -1;
+        }
     }
 
     if (xrootdfs.cns != NULL)
-        strcat(rootpath,xrootdfs.cns);
+        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
     else
-        strcat(rootpath,xrootdfs.rdr);
-    strcat(rootpath,path);
+        strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
+    strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid);
-    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
+    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
     xattrlen = XrdFfsPosix_getxattr(rootpath, name, value, size);
     if (xattrlen == -1)
         return -errno;
@@ -1159,6 +1166,7 @@ static void xrootdfs_usage(const char *progname)
 "    -o sss[=keytab]          use Xrootd seciruty module \"sss\", specifying a keytab file is optional\n"
 "    -o refreshdslist=NNNs/m/h/d  refresh internal list of data servers in NNN sec/min/hour/day, default unit is second\n"
 "                                 Absents of this option will disable automatically refreshing\n"
+"    -o maxfd=N               number of virtual file descriptors for posix requests, default 8192 (min 2048)\n"
 "    -o nworkers=N            number of workers to handle parallel requests to data servers, default 4\n"
 "    -o fastls=RDR            set to RDR when CNS is presented will cause stat() to go to redirector\n"
 "\n", progname);
@@ -1191,8 +1199,6 @@ static int xrootdfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
 
 int main(int argc, char *argv[])
 {
-    static XrdPosixXrootd abc; // Do one time init for posix interface
-
     xrootdfs_oper.init		= xrootdfs_init;
     xrootdfs_oper.getattr	= xrootdfs_getattr;
     xrootdfs_oper.access	= xrootdfs_access;
@@ -1224,14 +1230,15 @@ int main(int argc, char *argv[])
 /* Define XrootdFS options */
     char **cmdline_opts;
 
-    cmdline_opts = (char **) malloc(sizeof(char*) * (argc + 3));
+    cmdline_opts = (char **) malloc(sizeof(char*) * (argc -1 + 3));
     cmdline_opts[0] = argv[0];
     cmdline_opts[1] = strdup("-o");
     cmdline_opts[2] = strdup("fsname=xrootdfs,allow_other,max_write=131072,attr_timeout=10,entry_timeout=10,negative_timeout=5");
-    for (int i = 1; i <= argc; i++)
+
+    for (int i = 1; i < argc; i++)
         cmdline_opts[i+2] = argv[i];
 
-    struct fuse_args args = FUSE_ARGS_INIT(argc + 2, cmdline_opts);
+    struct fuse_args args = FUSE_ARGS_INIT(argc -1 + 3, cmdline_opts);
 
     xrootdfs_opts[0].templ = "-h";
     xrootdfs_opts[0].offset = -1U;
@@ -1288,7 +1295,12 @@ int main(int argc, char *argv[])
     xrootdfs_opts[11].offset = offsetof(struct XROOTDFS, nworkers);
     xrootdfs_opts[11].value = 0;
 
-    xrootdfs_opts[12].templ = NULL;
+/* number of virtual file descriptors */
+    xrootdfs_opts[12].templ = "maxfd=%d";
+    xrootdfs_opts[12].offset = offsetof(struct XROOTDFS, maxfd);
+    xrootdfs_opts[12].value = 0;
+
+    xrootdfs_opts[13].templ = NULL;
 
 /* initialize struct xrootdfs */
 //    memset(&xrootdfs, 0, sizeof(xrootdfs));
@@ -1300,6 +1312,7 @@ int main(int argc, char *argv[])
     xrootdfs.ssskeytab = NULL;
     xrootdfs.urlcachelife = strdup("3650d"); /* 10 years */
     xrootdfs.nworkers = 4;
+    xrootdfs.maxfd = 8192;
 
 /* Get options from environment variables first */
     xrootdfs.rdr = getenv("XROOTDFS_RDRURL");
@@ -1309,6 +1322,7 @@ int main(int argc, char *argv[])
     xrootdfs.daemon_user = getenv("XROOTDFS_USER");
     if (getenv("XROOTDFS_OFSFWD") != NULL && ! strcmp(getenv("XROOTDFS_OFSFWD"),"1")) xrootdfs.ofsfwd = true;
     if (getenv("XROOTDFS_NWORKERS") != NULL) sscanf(getenv("XROOTDFS_NWORKERS"), "%d", &xrootdfs.nworkers);
+    if (getenv("XROOTDFS_MAXFD") != NULL) sscanf(getenv("XROOTDFS_MAXFD"), "%d", &xrootdfs.maxfd);
 
 /* Parse XrootdFS options, will overwrite those defined in environment variables */
     fuse_opt_parse(&args, &xrootdfs, xrootdfs_opts, xrootdfs_opt_proc);
@@ -1341,13 +1355,13 @@ int main(int argc, char *argv[])
         setenv("XrdSecsssKT", xrootdfs.ssskeytab, 1);
     }
 
-    XrdFfsMisc_xrd_init(xrootdfs.rdr,xrootdfs.urlcachelife,0);
-    XrdFfsWcache_init();
+    if (xrootdfs.maxfd < 2048) xrootdfs.maxfd = 2048;
+
     signal(SIGUSR1,xrootdfs_sigusr1_handler);
 
     cwdfd = open(".",O_RDONLY);
-
     umask(0);
+
     return fuse_main(args.argc, args.argv, &xrootdfs_oper, NULL);
 }
 #else
