@@ -46,9 +46,6 @@
 
 
 using namespace XrdFileCache;
-namespace {
-static long long s_diskSpacePrecisionFactor = 10000000;
-}
 
 XrdVERSIONINFO(XrdOucGetCache, XrdFileCache);
 
@@ -121,7 +118,7 @@ bool Factory::xdlib(XrdOucStream &Config)
    std::string libp;
    if (!(val = Config.GetWord()) || !val[0])
    {
-      clLog()->Info(XrdCl::AppMsg, " Factory:;Config() decisionlib not specified; always caching files");
+      clLog()->Info(XrdCl::AppMsg, " Factory::Config() decisionlib not specified; always caching files");
       return true;
    }
    else
@@ -150,6 +147,7 @@ bool Factory::xdlib(XrdOucStream &Config)
       d->ConfigDecision(params);
 
    m_decisionpoints.push_back(d);
+   clLog()->Info(XrdCl::AppMsg, "Factory::Config() successfully created decision lib from %s", libp.c_str());
    return true;
 }
 //______________________________________________________________________________
@@ -211,6 +209,20 @@ bool Factory::Config(XrdSysLogger *logger, const char *config_filename, const ch
                                                 &XrdVERSIONINFOVAR(XrdOucGetCache));
    if (!ofsCfg) return false;
 
+
+   if (ofsCfg->Load(XrdOfsConfigPI::theOssLib))  {
+      ofsCfg->Plugin(m_output_fs);
+      XrdOssCache_FS* ocfs = XrdOssCache::Find("public");
+      ocfs->Add(m_configuration.m_cache_dir.c_str());
+   }
+   else
+   {
+      clLog()->Error(XrdCl::AppMsg, "Factory::Config() Unable to create an OSS object");
+      m_output_fs = 0;
+      return false;
+   }
+
+
    // Actual parsing of the config file.
    bool retval = true;
    char *var;
@@ -239,23 +251,19 @@ bool Factory::Config(XrdSysLogger *logger, const char *config_filename, const ch
    }
 
    Config.Close();
-
+   // sets default value for disk usage
+   if (m_configuration.m_diskUsageLWM < 0 || m_configuration.m_diskUsageHWM < 0)
+   {
+      XrdOssVSInfo sP;
+      if (m_output_fs->StatVS(&sP, "public", 1) >= 0) {
+         m_configuration.m_diskUsageLWM = static_cast<long long>(0.90 * sP.Total + 0.5);
+         m_configuration.m_diskUsageHWM = static_cast<long long>(0.95 * sP.Total + 0.5);
+         clLog()->Debug(XrdCl::AppMsg, "Default disk usage [%lld, %lld]", m_configuration.m_diskUsageLWM, m_configuration.m_diskUsageHWM);
+      }
+   }
 
    if (retval)
    {
-      if (ofsCfg->Load(XrdOfsConfigPI::theOssLib))  {
-          ofsCfg->Plugin(m_output_fs);
-          XrdOssCache_FS* ocfs = XrdOssCache::Find("public");
-          ocfs->Add(m_configuration.m_cache_dir.c_str());
-      }
-      else
-      {
-         clLog()->Error(XrdCl::AppMsg, "Factory::Config() Unable to create an OSS object");
-         retval = false;
-         m_output_fs = 0;
-      }
-
-  
       int loff = 0;
       char buff[2048];
       loff = snprintf(buff, sizeof(buff), "result\n"
@@ -309,17 +317,39 @@ bool Factory::ConfigParameters(std::string part, XrdOucStream& config )
    }
    else if  ( part == "diskusage" )
    {
-      const char* minV = config.GetWord();
-      if (minV) {
-         m_configuration.m_lwm = ::atof(minV);
-         const char* maxV = config.GetWord();
-         if (maxV) {
-            m_configuration.m_hwm = ::atof(maxV);
+      std::string minV = config.GetWord();
+      std::string maxV = config.GetWord();
+      if (!minV.empty() && !maxV.empty()) {
+         XrdOssVSInfo sP;
+         if (m_output_fs->StatVS(&sP, "public", 1) >= 0)
+         {
+            if (::isalpha(*(minV.rbegin())) && ::isalpha(*(minV.rbegin()))) {
+               if ( XrdOuca2x::a2sz(m_log, "Error getting disk usage low watermark",  minV.c_str(), &m_configuration.m_diskUsageLWM, 0, sP.Total) 
+                 || XrdOuca2x::a2sz(m_log, "Error getting disk usage high watermark", maxV.c_str(), &m_configuration.m_diskUsageHWM, 0, sP.Total))
+               {
+                  return false;
+               }
+            }
+            else 
+            {
+               char* eP;
+               errno = 0;
+               float lwmf = strtod(minV.c_str(), &eP);
+               if (errno || eP == minV.c_str()) {
+                    m_log.Emsg("Factory::ConfigParameters() error parsing diskusage parameter ", minV.c_str());
+                    return false;
+               }
+               float hwmf = strtod(maxV.c_str(), &eP);
+               if (errno || eP == maxV.c_str()) {
+                  m_log.Emsg("Factory::ConfigParameters() error parsing diskusage parameter ", maxV.c_str());
+                  return false;
+               }
+
+               m_configuration.m_diskUsageLWM = static_cast<long long>(sP.Total * lwmf + 0.5);
+               m_configuration.m_diskUsageHWM = static_cast<long long>(sP.Total * hwmf + 0.5);
+            }
          }
       }
-      else {
-         clLog()->Error(XrdCl::AppMsg, "Factory::ConfigParameters() pss.diskUsage min max value not specified");
-      }
    }
    else if  ( part == "blocksize" )
    {
@@ -373,41 +403,46 @@ bool Factory::ConfigParameters(std::string part, XrdOucStream& config )
 //______________________________________________________________________________
 //namespace {
 
-class FPurgeState {
+class FPurgeState
+{
 public:
-   struct FS {
+   struct FS
+   {
       std::string path;
-      int nBlck;
+      long long   nByte;
 
-      FS(const char* p, int n) : path(p), nBlck(n) {}
+      FS(const char* p, long long n) : path(p), nByte(n) {}
    };
 
    typedef std::multimap<time_t, FS> map_t;
    typedef map_t::iterator map_i;
 
-   FPurgeState(long long iNBlckReq) : nBlckReq(iNBlckReq), nBlckAccum(0) {}
+   FPurgeState(long long iNByteReq) : nByteReq(iNByteReq), nByteAccum(0) {}
 
    map_t fmap;
 
-   void checkFile (time_t iTime, const char* iPath,  int iNBlck) 
+   void checkFile (time_t iTime, const char* iPath,  long long iNByte)
    {
-      if ( (nBlckAccum <  nBlckReq ) || (iTime < fmap.rbegin()->first) ) {
-         fmap.insert(std::pair<const time_t, FS> (iTime, FS(iPath, iNBlck)));
-         nBlckAccum += iNBlck;
+      if (nByteAccum < nByteReq || iTime < fmap.rbegin()->first)
+      {
+         fmap.insert(std::pair<const time_t, FS> (iTime, FS(iPath, iNByte)));
+         nByteAccum += iNByte;
 
          // remove newest files from map if necessary
-         while (nBlckAccum > nBlckReq) {
+         while (nByteAccum > nByteReq)
+         {
             time_t nt = fmap.begin()->first;
             std::pair<map_i, map_i> ret = fmap.equal_range(nt); 
             for (map_i it2 = ret.first; it2 != ret.second; ++it2)
-               nBlckAccum -= it2->second.nBlck;
+               nByteAccum -= it2->second.nByte;
+	    fmap.erase(ret.first, ret.second);
          }
       }
    }
 
-   private:
-      long long nBlckReq;
-      long long nBlckAccum;
+private:
+   long long nByteReq;
+   long long nByteAccum;
 };
 
 
@@ -422,12 +457,12 @@ void FillFileMapRecurse( XrdOssDF* iOssDF, const std::string& path, FPurgeState&
    XrdCl::Log *log = XrdCl::DefaultEnv::GetLog();
 
    Factory& factory = Factory::GetInstance();
-   while ( (rdr = iOssDF->Readdir(&buff[0], 256)) >= 0)
+   while ((rdr = iOssDF->Readdir(&buff[0], 256)) >= 0)
    {
       // printf("readdir [%s]\n", buff);
       std::string np = path + "/" + std::string(buff);
       size_t fname_len = strlen(&buff[0]);
-      if (fname_len == 0  )
+      if (fname_len == 0)
       {
          // std::cout << "Finish read dir.[" << np <<"] Break loop \n";
          break;
@@ -440,21 +475,48 @@ void FillFileMapRecurse( XrdOssDF* iOssDF, const std::string& path, FPurgeState&
 
          if (fname_len > InfoExtLen && strncmp(&buff[fname_len - InfoExtLen ], XrdFileCache::Info::m_infoExtension, InfoExtLen) == 0)
          {
-            fh->Open((np).c_str(),O_RDONLY, 0600, env);
+            // XXXX MT - shouldn't we also check if it is currently opened?
+
+            fh->Open(np.c_str(), O_RDONLY, 0600, env);
             Info cinfo(factory.RefConfiguration().m_bufferSize);
             time_t accessTime;
             cinfo.Read(fh);
             if (cinfo.GetLatestDetachTime(accessTime, fh))
             {
                log->Debug(XrdCl::AppMsg, "FillFileMapRecurse() checking %s accessTime %d ", buff, (int)accessTime);
-               purgeState.checkFile(accessTime, np.c_str(), cinfo.GetNDownloadedBlocks());
+               purgeState.checkFile(accessTime, np.c_str(), cinfo.GetNDownloadedBytes());
             }
             else
             {
-               log->Warning(XrdCl::AppMsg, "FillFileMapRecurse() could not get access time for %s \n", np.c_str());
+               // cinfo file does not contain any known accesses, use stat.mtime instead.
+
+               log->Info(XrdCl::AppMsg, "FillFileMapRecurse() could not get access time for %s, trying stat.\n", np.c_str());
+
+               XrdOss* oss = Factory::GetInstance().GetOss();
+               struct stat fstat;
+
+               if (oss->Stat(np.c_str(), &fstat) == XrdOssOK)
+               {
+                  accessTime = fstat.st_mtime;
+                  log->Info(XrdCl::AppMsg, "FillFileMapRecurse() determined access time for %s via stat: %lld\n",
+                                                np.c_str(), accessTime);
+
+                  purgeState.checkFile(accessTime, np.c_str(), cinfo.GetNDownloadedBytes());
+               }
+               else
+               {
+                  // This really shouldn't happen ... but if it does remove cinfo and the data file right away.
+
+                  log->Warning(XrdCl::AppMsg, "FillFileMapRecurse() could not get access time for %s. Purging directly.\n",
+                               np.c_str());
+
+                  oss->Unlink(np.c_str());
+                  np = np.substr(0, np.size() - strlen(XrdFileCache::Info::m_infoExtension));
+                  oss->Unlink(np.c_str());
+               }
             }
          }
-         else if ( dh->Opendir(np.c_str(), env)  >= 0 )
+         else if (dh->Opendir(np.c_str(), env) >= 0)
          {
             FillFileMapRecurse(dh, np, purgeState);
          }
@@ -473,64 +535,64 @@ void Factory::CacheDirCleanup()
    struct stat fstat;
    XrdOucEnv env;
 
-   XrdOss* oss =  Factory::GetInstance().GetOss();
+   XrdOss* oss = Factory::GetInstance().GetOss();
    XrdOssVSInfo sP;
 
    while (1)
    {
       // get amount of space to erase
       long long bytesToRemove = 0;
-      if( oss->StatVS(&sP, "public", 1) < 0 )
+      if (oss->StatVS(&sP, "public", 1) < 0)
       {
          clLog()->Error(XrdCl::AppMsg, "Factory::CacheDirCleanup() can't get statvs for dir [%s] \n", m_configuration.m_cache_dir.c_str());
          exit(1);
       }
       else
       {
-         float oc = 1 - float(sP.Free)/(sP.Total);
-         clLog()->Debug(XrdCl::AppMsg, "Factory::CacheDirCleanup() occupates disk space == %f", oc);
-         if (oc > m_configuration.m_hwm)
+         long long ausage = sP.Total - sP.Free;
+         clLog()->Info(XrdCl::AppMsg, "Factory::CacheDirCleanup() occupates disk space == %lld", ausage);
+         if (ausage > m_configuration.m_diskUsageHWM)
          {
-            long long bytesToRemoveLong = static_cast<long long> ((oc - m_configuration.m_lwm) * static_cast<float>(s_diskSpacePrecisionFactor));
-            bytesToRemove = (sP.Total * bytesToRemoveLong) / s_diskSpacePrecisionFactor;
+            bytesToRemove = ausage - m_configuration.m_diskUsageLWM;
             clLog()->Info(XrdCl::AppMsg, "Factory::CacheDirCleanup() need space for  %lld bytes", bytesToRemove);
          }
       }
 
       if (bytesToRemove > 0)
       {
-        // make a sorted map of file patch by access time
+         // make a sorted map of file patch by access time
          XrdOssDF* dh = oss->newDir(m_configuration.m_username.c_str());
          if (dh->Opendir(m_configuration.m_cache_dir.c_str(), env) >= 0)
          {
-            long long nReq = (long long) ((bytesToRemove*1.4)/m_configuration.m_bufferSize); // check more that required
-            FPurgeState purgeState(nReq);
+            FPurgeState purgeState(bytesToRemove * 5 / 4); // prepare 20% more volume than required
+
             FillFileMapRecurse(dh, m_configuration.m_cache_dir, purgeState);
 
             // loop over map and remove files with highest value of access time
             for (FPurgeState::map_i it = purgeState.fmap.begin(); it != purgeState.fmap.end(); ++it)
             {
-               std::pair<FPurgeState::map_i, FPurgeState::map_i> ret = purgeState.fmap.equal_range(it->first); 
-               for (FPurgeState::map_i it2 = ret.first; it2 != ret.second; ++it2)
-               {
-                  std::string path = it2->second.path;
-                  // remove info file
-                  if (oss->Stat(path.c_str(), &fstat) == XrdOssOK)
-                  {
-                     bytesToRemove -= fstat.st_size;
-                     oss->Unlink(path.c_str());
-                     clLog()->Info(XrdCl::AppMsg, "Factory::CacheDirCleanup() removed %s size %lld ", path.c_str(), fstat.st_size);
-                  }
+               // XXXX MT - shouldn't we re-check if the file is currently opened?
 
-                  // remove data file
-                  path = path.substr(0, path.size() - strlen(XrdFileCache::Info::m_infoExtension));
-                  if (oss->Stat(path.c_str(), &fstat) == XrdOssOK)
-                  {
-                     bytesToRemove -= fstat.st_size;
-                     oss->Unlink(path.c_str());
-                     clLog()->Info(XrdCl::AppMsg, "Factory::CacheDirCleanup() removed %s size %lld ", path.c_str(), fstat.st_size);
-                  }
+               std::string path = it->second.path;
+               // remove info file
+               if (oss->Stat(path.c_str(), &fstat) == XrdOssOK)
+               {
+                  bytesToRemove -= fstat.st_size;
+                  oss->Unlink(path.c_str());
+                  clLog()->Info(XrdCl::AppMsg, "Factory::CacheDirCleanup() removed %s size %lld",
+                                                path.c_str(), fstat.st_size);
                }
+
+               // remove data file
+               path = path.substr(0, path.size() - strlen(XrdFileCache::Info::m_infoExtension));
+               if (oss->Stat(path.c_str(), &fstat) == XrdOssOK)
+               {
+                  bytesToRemove -= it->second.nByte;
+                  oss->Unlink(path.c_str());
+                  clLog()->Info(XrdCl::AppMsg, "Factory::CacheDirCleanup() removed %s bytes %lld, stat_size %lld",
+                                                path.c_str(), it->second.nByte, fstat.st_size);
+               }
+
                if (bytesToRemove <= 0)
                   break;
             }
@@ -538,6 +600,7 @@ void Factory::CacheDirCleanup()
 	 dh->Close();
 	 delete dh; dh =0;
       }
+
       sleep(sleept);
    }
 }
