@@ -35,20 +35,19 @@
 #include <fcntl.h>
 #include <sys/param.h>
 #include <sys/types.h>
-#include <sys/socket.h>
 #include <sys/stat.h>
 
 #include "XrdVersion.hh"
 
 #include "Xrd/XrdInfo.hh"
-#include "XrdCks/XrdCksConfig.hh"
-#include "XrdCks/XrdCksManager.hh"
 #include "XrdFrc/XrdFrcTrace.hh"
 #include "XrdFrc/XrdFrcUtils.hh"
 #include "XrdFrm/XrdFrmCns.hh"
 #include "XrdFrm/XrdFrmConfig.hh"
 #include "XrdFrm/XrdFrmMonitor.hh"
+#include "XrdNet/XrdNetAddr.hh"
 #include "XrdNet/XrdNetCmsNotify.hh"
+#include "XrdOfs/XrdOfsConfigPI.hh"
 #include "XrdOss/XrdOss.hh"
 #include "XrdOss/XrdOssSpace.hh"
 #include "XrdOuc/XrdOuca2x.hh"
@@ -63,13 +62,13 @@
 #include "XrdOuc/XrdOucTList.hh"
 #include "XrdOuc/XrdOucTokenizer.hh"
 #include "XrdOuc/XrdOucUtils.hh"
-#include "XrdSys/XrdSysDNS.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysHeaders.hh"
 #include "XrdSys/XrdSysLogger.hh"
 #include "XrdSys/XrdSysTimer.hh"
 #include "XrdSys/XrdSysPlatform.hh"
 #include "XrdSys/XrdSysPthread.hh"
+#include "XrdSys/XrdSysUtils.hh"
 
 using namespace XrdFrc;
 using namespace XrdFrm;
@@ -129,18 +128,6 @@ void *XrdFrmConfigMum(void *parg)
    return (void *)0;
 }
 
-void *XrdLogWorker(void *parg)
-{
-   char *mememe = strdup((char *)parg);
-
-   while(1)
-        {XrdSysTimer::Wait4Midnight();
-         Say.Say(0, XrdBANNER);
-         Say.Say(0, mememe, " running.");
-        }
-   return (void *)0;
-}
-
 /******************************************************************************/
 /*                           C o n s t r u c t o r                            */
 /******************************************************************************/
@@ -174,8 +161,7 @@ XrdFrmConfig::XrdFrmConfig(SubSys ss, const char *vopts, const char *uinfo)
    xfrCmd[2].Desc = "copycmd in url"; xfrCmd[3].Desc = "copycmd out url";
    xfrIN    = xfrOUT = 0;
    isAgent  = (getenv("XRDADMINPATH") ? 1 : 0);
-   ossLib   = 0;
-   ossParms = 0;
+   OfsCfg   = 0;
    cmsPath  = 0;
    haveCMS  = 0;
    isOTO    = 0;
@@ -193,6 +179,7 @@ XrdFrmConfig::XrdFrmConfig(SubSys ss, const char *vopts, const char *uinfo)
    runOld   = 0;
    runNew   = 1;
    nonXA    = 0;
+   doStatPF = 0;
 
    myUid    = geteuid();
    myGid    = getegid();
@@ -200,8 +187,6 @@ XrdFrmConfig::XrdFrmConfig(SubSys ss, const char *vopts, const char *uinfo)
    LocalRoot= RemoteRoot = 0;
    lcl_N2N  = rmt_N2N = the_N2N = 0;
    N2N_Lib  = N2N_Parms         = 0;
-   CksAlg   = 0;
-   CksCfg   = 0;
    CksMan   = 0;
 
    xfrFdir  = 0;
@@ -245,16 +230,16 @@ XrdFrmConfig::XrdFrmConfig(SubSys ss, const char *vopts, const char *uinfo)
 int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
 {
    extern XrdOss *XrdOssGetSS(XrdSysLogger *, const char *, const char *,
-                              const char   *, XrdVersionInfo &);
+                              const char   *, XrdOucEnv *,  XrdVersionInfo &);
    extern int *XrdOssRunMode;
+   static XrdNetAddr myAddr(0);
    XrdFrmConfigSE theSE;
-   int n, retc, isMum = 0, myXfrMax = -1, NoGo = 0, optBG = 0;
+   int retc, isMum = 0, myXfrMax = -1, NoGo = 0, optBG = 0;
    const char *temp;
    char c, buff[1024], *logfn = 0;
-   long long logkeep = 0;
    extern char *optarg;
    extern int opterr, optopt;
-   int pipeFD[2] = {-1, -1};
+   int pipeFD[2] = {-1, -1}, bindArg = -1, pureLFN = 0;
    const char *pidFN = 0;
 
 // Obtain the program name (used for logging)
@@ -268,7 +253,7 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
 //
    opterr = 0; nextArg = 1;
    while(nextArg < argc && '-' == *argv[nextArg]
-         && (c=getopt(argc,argv,vOpts)) && (c != -1))
+         && (c=getopt(argc,argv,vOpts)) && (c != (char)-1))
      { switch(c)
        {
        case 'b': optBG = 1;
@@ -282,14 +267,17 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
        case 'f': Fix = 1;
                  break;
        case 'h': Usage(0);
-       case 'k': n = strlen(optarg)-1;
-                 retc = (isalpha(optarg[n])
-                        ? XrdOuca2x::a2sz(Say,"keep size", optarg,&logkeep)
-                        : XrdOuca2x::a2ll(Say,"keep count",optarg,&logkeep));
-                 if (retc) Usage(1);
-                 if (!isalpha(optarg[n])) logkeep = -logkeep;
+       case 'k': if (!(bindArg = Say.logger()->ParseKeep(optarg)))
+                    {Say.Emsg("Config","Invalid -k argument -",optarg);
+                     Usage(1);
+                    }
                  break;
-       case 'l': if (logfn) free(logfn);
+       case 'l': if ((pureLFN = *optarg == '=')) optarg++;
+                 if (!*optarg)
+                    {Say.Emsg("Config", "Logfile name not specified.");
+                     Usage(1);
+                    }
+                 if (logfn) free(logfn);
                  logfn = strdup(optarg);
                  break;
        case 'm': if (XrdOuca2x::a2i(Say,"max number",optarg,&myXfrMax))
@@ -312,6 +300,8 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
                  break;
        case 'S': mySite= optarg;
                  break;
+       case 'z': Say.logger()->setHiRes();
+                 break;
        default:  sprintf(buff,"'%c'", optopt);
                  if (c == ':') Say.Emsg("Config", buff, "value not specified.");
                     else Say.Emsg("Config", buff, "option is invalid");
@@ -334,7 +324,8 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
                                            (isAgent ? 'a' : 'd'));
                logfn = strdup(buff);
               }
-          } else if (!(logfn=XrdOucUtils::subLogfn(Say,myInst,logfn))) _exit(16);
+          } else if (!pureLFN
+                 && !(logfn=XrdOucUtils::subLogfn(Say,myInst,logfn))) _exit(16);
 
    // If undercover desired and we are not an agent, do so
    //
@@ -352,14 +343,14 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
    // Bind the log file if we have one
    //
        if (logfn)
-          {if (logkeep) Say.logger()->setKeep(logkeep);
-           Say.logger()->Bind(logfn, 24*60*60);
+          {Say.logger()->AddMsg(XrdBANNER);
+           if (Say.logger()->Bind(logfn, bindArg)) _exit(19);
           }
       }
 
 // Get the full host name. In theory, we should always get some kind of name.
 //
-   if (!(myName = XrdSysDNS::getHostName()))
+   if (!(myName = myAddr.Name()))
       {Say.Emsg("Config","Unable to determine host name; execution terminated.");
        _exit(16);
       }
@@ -380,6 +371,14 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
    if (!logfn && (ssID == ssAdmin || isOTO) && !Trace.What)
       isMum = ConfigMum(theSE);
 
+// Add final message to the logger
+//
+   if (logfn)
+      {char msgBuff[2048];
+       strcpy(msgBuff, myInstance); strcat(msgBuff, " running.");
+       Say.logger()->AddMsg(msgBuff);
+      }
+
 // Put out the herald
 //
    sprintf(buff, "Scalla %s is starting. . .", myProg);
@@ -404,9 +403,15 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
        XrdOucEnv::Export("XRDOSSTYPE",  myFrmID);
        if (ssID == ssPurg) XrdOucEnv::Export("XRDOSSCSCAN", "off");
        if (!NoGo)
-          {if (!(ossFS=XrdOssGetSS(Say.logger(), ConfigFN, ossLib, ossParms,
-                                   *myVersion))) NoGo = 1;
-              else runNew = !(runOld = XrdOssRunMode ? *XrdOssRunMode : 0);
+          {int loadPI = XrdOfsConfigPI::theOssLib | XrdOfsConfigPI::theAtrLib;
+           if (ssID == ssAdmin) loadPI |= XrdOfsConfigPI::theCksLib;
+           if (!OfsCfg->Load(loadPI)) NoGo = 1;
+              else {struct stat Stat;
+                    OfsCfg->Plugin(ossFS);
+                    OfsCfg->Plugin(CksMan);
+                    doStatPF = ossFS->StatPF("/", &Stat) != -ENOTSUP;
+                    runNew = !(runOld = XrdOssRunMode ? *XrdOssRunMode : 0);
+                   }
           }
       }
 
@@ -417,7 +422,7 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
 // Configure each specific component
 //
    if (!NoGo) switch(ssID)
-      {case ssAdmin: NoGo = (ConfigN2N() || ConfigMss() || ConfigCks());
+      {case ssAdmin: NoGo = (ConfigN2N() || ConfigMss());
                      break;
        case ssPurg:  if (!(NoGo = (ConfigMon(0) || ConfigMP("purgeable"))))
                         ConfigPF("frm_purged");
@@ -432,15 +437,6 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
 //
    if (!NoGo && ppf) NoGo = ppf();
 
-// Start the log turn-over thread
-//
-   if (!NoGo && logfn)
-      {pthread_t tid;
-       if ((retc = XrdSysThread::Run(&tid, XrdLogWorker, (void *)myInstance,
-                                     XRDSYSTHREAD_BIND, "midnight runner")))
-          {Say.Emsg("Config", retc, "create logger thread"); NoGo = 1;}
-      }
-
    // if we call this it means that the daemon has forked and we are
    // in the child process
 #ifndef WIN32
@@ -470,6 +466,7 @@ int XrdFrmConfig::Configure(int argc, char **argv, int (*ppf)())
 
 // All done
 //
+   if (OfsCfg) delete OfsCfg;
    return !NoGo;
 }
   
@@ -583,25 +580,19 @@ XrdOucTList *XrdFrmConfig::Space(const char *Name, const char *Path)
    return (tP ? tP : &nullEnt);
 }
 
+/******************************************************************************/
+/*                                  S t a t                                   */
+/******************************************************************************/
+  
+int XrdFrmConfig::Stat(const char *xLfn, const char *xPfn, struct stat *buff)
+{
+   return (doStatPF ? ossFS->StatPF(xPfn, buff)
+                    : ossFS->Stat  (xLfn, buff, XRDOSS_resonly));
+}
+
 /******************************************************************************/
 /*                     P r i v a t e   F u n c t i o n s                      */
 /******************************************************************************/
-/******************************************************************************/
-/*                             C o n f i g C k s                              */
-/******************************************************************************/
-
-int XrdFrmConfig::ConfigCks()
-{
-
-// If we have no algorithm, we are done
-//
-   if (!CksAlg) return 0;
-
-// Configure the algorithm
-//
-   return !(CksMan = CksCfg->Configure(CksAlg));
-}
-  
 /******************************************************************************/
 /* Private:                    C o n f i g C m d                              */
 /******************************************************************************/
@@ -735,6 +726,13 @@ int XrdFrmConfig::ConfigMP(const char *pType)
 //
    while((tP = expList)) {expList = tP->next; delete tP;}
 
+// For purging, make sure we have at least one path to purge
+//
+   if (xOpt == XRDEXP_PURGE && !pathList)
+      {Say.Emsg("Config","No purgeable paths specified!");
+       NoGo = 1;
+      }
+
 // The oss would have already set NORCREATE and NOCHECK for all stageable paths.
 // But now, we must also off the R/O flag on every purgeable and stageable path
 // to prevent oss complaints. This needs to be defered to here because we need
@@ -960,12 +958,10 @@ int XrdFrmConfig::ConfigProc()
   XrdOucEnv myEnv;
   XrdOucStream cfgFile(&Say, myInstance, &myEnv, "=====> ");
 
-// Allocate a chksum configurator if needed
+// Allocate a plugin library configurator
 //
-   if (ssID == ssAdmin)
-      {CksCfg = new XrdCksConfig(ConfigFN, &Say, retc, *myVersion);
-       if (!retc) return 1;
-      }
+   OfsCfg = XrdOfsConfigPI::New(ConfigFN, &cfgFile, &Say, myVersion);
+   if (!OfsCfg) return 1;
 
 // Try to open the configuration file.
 //
@@ -998,6 +994,8 @@ int XrdFrmConfig::ConfigProc()
 /* Prvate:                     C o n f i g X e q                              */
 /******************************************************************************/
 
+#define PARSEPI(x) return !OfsCfg->Parse(XrdOfsConfigPI:: x);
+
 int XrdFrmConfig::ConfigXeq(char *var, int mbok)
 {
 
@@ -1013,8 +1011,9 @@ int XrdFrmConfig::ConfigXeq(char *var, int mbok)
    if (ssID == ssAdmin)
       {
        if (!strcmp(var, "frm.xfr.qcheck")) return xqchk();
-       if (!strcmp(var, "ofs.ckslib"    )) return xcks(1);
-       if (!strcmp(var, "ofs.osslib"    )) return xoss();
+       if (!strcmp(var, "ofs.ckslib"    )) PARSEPI(theCksLib);
+       if (!strcmp(var, "ofs.osslib"    )) PARSEPI(theOssLib);
+       if (!strcmp(var, "ofs.xattrlib"  )) PARSEPI(theAtrLib);
        if (!strcmp(var, "oss.cache"     )){hasCache = 1; // runOld
                                            return xspace(0,0);
                                           }
@@ -1034,7 +1033,8 @@ int XrdFrmConfig::ConfigXeq(char *var, int mbok)
        if (isAgent) return 0;           // Server-oriented directives
 
        if (!strcmp(var, "all.sitename"  )) return xsit();
-       if (!strcmp(var, "ofs.osslib"    )) return Grab(var, &ossLib,    0);
+       if (!strcmp(var, "ofs.osslib"    )) PARSEPI(theOssLib);
+       if (!strcmp(var, "ofs.xattrlib"  )) PARSEPI(theAtrLib);
        if (!strcmp(var, "oss.cache"     )) return xspace(0,0);
        if (!strcmp(var, "oss.localroot" )) return Grab(var, &LocalRoot, 0);
        if (!strcmp(var, "oss.namelib"   )) return xnml();
@@ -1059,7 +1059,8 @@ int XrdFrmConfig::ConfigXeq(char *var, int mbok)
        if (!strcmp(var, "dirhold"       )) return xdpol();
        if (!strcmp(var, "oss.cache"     )) return xspace(1,0);
        if (!strcmp(var, "oss.localroot" )) return Grab(var, &LocalRoot, 0);
-       if (!strcmp(var, "ofs.osslib"    )) return Grab(var, &ossLib,    0);
+       if (!strcmp(var, "ofs.osslib"    )) PARSEPI(theOssLib);
+       if (!strcmp(var, "ofs.xattrlib"  )) PARSEPI(theAtrLib);
        if (!strcmp(var, "policy"        )) return xpol();
        if (!strcmp(var, "polprog"       )) return xpolprog();
        if (!strcmp(var, "oss.space"     )) return xspace(1);
@@ -1199,7 +1200,7 @@ int XrdFrmConfig::Grab(const char *var, char **Dest, int nosubs)
 XrdOucTList *XrdFrmConfig::InsertPL(XrdOucTList *pL, const char *Path,
                                     int Plen, int isRW)
 {
-   short sval[4] = {(short)isRW, (short)Plen};
+   short sval[4] = {static_cast<short>(isRW), static_cast<short>(Plen)};
    XrdOucTList *pP = 0, *tP = pL;
 
 // Find insertion point
@@ -1311,14 +1312,10 @@ int XrdFrmConfig::xapath()
   Output: 0 upon success or !0 upon failure.
 */
 
-int XrdFrmConfig::xcks(int isOfs)
+int XrdFrmConfig::xcks()
 {
    char *palg;
 
-// If this is the ofs cks directive, then process as such
-//
-   if (isOfs) return CksCfg->ParseLib(*cFile);
-
 // Get the algorithm name and the program implementing it
 //
    while ((palg = cFile->GetWord()) && *palg != '/')
@@ -1331,8 +1328,10 @@ int XrdFrmConfig::xcks(int isOfs)
 //
    if (!palg || *palg == '/')
       {Say.Emsg("Config", "chksum algorithm not specified"); return 1;}
-   if (CksAlg) free(CksAlg);
-   CksAlg = strdup(palg);
+
+// Set default checksum
+//
+   OfsCfg->DefaultCS(palg);
    return 0;
 }
   
@@ -1355,7 +1354,7 @@ int XrdFrmConfig::xcks(int isOfs)
 */
 int XrdFrmConfig::xcnsd()
 {
-   int cnsMode;
+   int cnsMode = 0;
    char *val, *cnsPath = 0;
    struct cnsdopts {const char *opname; int opval;} cnsopt[] =
          {
@@ -1399,11 +1398,13 @@ int XrdFrmConfig::xcnsd()
 
    Purpose:  To parse the directive: copycmd [Options] cmd [args]
 
-   Options:  [in] [out] [stats] [timeout <sec>] [url] [xpd] cmd [args]
+   Options:  [in] [noalloc] [out] [rmerr] [stats] [timeout <sec>] [url] [xpd]
 
              in        use command for incomming copies.
              noalloc   do not pre-allocate space for incomming copies.
              out       use command for outgoing copies.
+             rmerr     remove incomming file when copy ends with an error.
+                       Default unless noalloc is specified.
              stats     print transfer statistics in the log.
              timeout   how long the cmd can run before it is killed.
              url       use command for url-based transfers.
@@ -1412,7 +1413,7 @@ int XrdFrmConfig::xcnsd()
    Output: 0 upon success or !0 upon failure.
 */
 int XrdFrmConfig::xcopy()
-{  int cmdIO[2] = {0,0}, TLim=0, Stats=0, hasMDP=0, cmdUrl=0, noAlo=0;
+{  int cmdIO[2] = {0,0}, TLim=0, Stats=0, hasMDP=0, cmdUrl=0, noAlo=0, rmErr=0;
    int monPD = 0;
    char *val, *theCmd = 0;
    struct copyopts {const char *opname; int *oploc;} cpopts[] =
@@ -1420,6 +1421,7 @@ int XrdFrmConfig::xcopy()
           {"in",     &cmdIO[0]},
           {"out",    &cmdIO[1]},
           {"noalloc",&noAlo},
+          {"rmerr",  &rmErr},
           {"stats",  &Stats},
           {"timeout",&TLim},
           {"url",    &cmdUrl},
@@ -1463,6 +1465,7 @@ int XrdFrmConfig::xcopy()
            if (Stats)  xfrCmd[n].Opts  |= cmdStats;
            if (monPD)  xfrCmd[n].Opts  |= cmdXPD;
            if (hasMDP) xfrCmd[n].Opts  |= cmdMDP;
+           if (rmErr)  xfrCmd[n].Opts  |= cmdRME;
            if (noAlo)  xfrCmd[n].Opts  &=~cmdAlloc;
               else     xfrCmd[n].Opts  |= cmdAlloc;
            xfrCmd[n].TLimit = TLim;
@@ -1678,47 +1681,13 @@ int XrdFrmConfig::xnml()
    return 0;
 }
 
-/******************************************************************************/
-/* Private:                         x o s s                                   */
-/******************************************************************************/
-  
-
-/* Function: xoss
-
-   Purpose:  To parse the directive: osslib <path>
-
-             <path>    the path of the filesystem library to be used.
-
-  Output: 0 upon success or !0 upon failure.
-*/
-
-int XrdFrmConfig::xoss()
-{
-    char *val, parms[1024];
-
-// Get the path
-//
-   if (!(val = cFile->GetWord()) || !val[0])
-      {Say.Emsg("Config", "osslib not specified"); return 1;}
-   if (ossLib) free(ossLib);
-   ossLib = strdup(val);
-
-// Record any parms
-//
-   if (!cFile->GetRest(parms, sizeof(parms)))
-      {Say.Emsg("Config", "osslib parameters too long"); return 1;}
-   if (ossParms) free(ossParms);
-   ossParms = (*parms ? strdup(parms) : 0);
-   return 0;
-}
-
 /******************************************************************************/
 /* Private:                         x p o l                                   */
 /******************************************************************************/
 
 /* Function: xpol
 
-   Purpose:  To parse the directive: policy {*|sname} {nopurge|min [max]] [opts]
+   Purpose:  To parse the directive: policy {*|sname} {nopurge|min} max]] [opts]
 
              *         The default policy for all spaces.
 
