@@ -42,9 +42,9 @@
 #include "XrdVersion.hh"
 
 #include "XrdCks/XrdCks.hh"
-#include "XrdCks/XrdCksConfig.hh"
 
 #include "XrdOfs/XrdOfs.hh"
+#include "XrdOfs/XrdOfsConfigPI.hh"
 #include "XrdOfs/XrdOfsEvs.hh"
 #include "XrdOfs/XrdOfsPoscq.hh"
 #include "XrdOfs/XrdOfsStats.hh"
@@ -57,13 +57,15 @@
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysHeaders.hh"
-#include "XrdSys/XrdSysPlugin.hh"
 #include "XrdOuc/XrdOucStream.hh"
 #include "XrdOuc/XrdOucTrace.hh"
 #include "XrdOuc/XrdOucUtils.hh"
 
+#include "XrdNet/XrdNetAddr.hh"
+
 #include "XrdCms/XrdCmsClient.hh"
 #include "XrdCms/XrdCmsFinder.hh"
+#include "XrdCms/XrdCmsRole.hh"
 
 #include "XrdAcc/XrdAccAuthorize.hh"
 
@@ -86,6 +88,9 @@ XrdVERSIONINFO(XrdOfs,XrdOfs);
 
 #define TS_Xeq(x,m)   if (!strcmp(x,var)) return m(Config,Eroute);
 
+#define TS_XPI(x,m)   if (!strcmp(x,var))\
+                         return !ofsConfig->Parse(XrdOfsConfigPI:: m);
+
 #define TS_Str(x,m)   if (!strcmp(x,var)) {free(m); m = strdup(val); return 0;}
 
 #define TS_PList(x,m)  if (!strcmp(x,var)) \
@@ -117,8 +122,8 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
 
   Output:   0 upon success or !0 otherwise.
 */
-   extern XrdOss *XrdOssGetSS(XrdSysLogger *, const char *, const char *,
-                                              const char *, XrdVersionInfo &);
+//?extern XrdOss *XrdOssGetSS(XrdSysLogger *, const char *, const char *,
+//?                           const char   *, XrdOucEnv  *, XrdVersionInfo &);
    char *var;
    const char *tmp;
    int  i, j, cfgFD, retc, NoGo = 0;
@@ -129,16 +134,21 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
 //
    Eroute.Say("++++++ File system initialization started.");
 
+// Establish the network interface that the caller must provide
+//
+   if (!EnvInfo || !(myIF = (XrdNetIF *)EnvInfo->GetPtr("XrdNetIF*")))
+      {Eroute.Emsg("Finder", "Network i/f undefined; unable to self-locate.");
+       NoGo = 1;
+      }
+
 // Preset all variables with common defaults
 //
    Options            = 0;
    if (getenv("XRDDEBUG")) OfsTrace.What = TRACE_MOST | TRACE_debug;
 
-// Allocate a checksum configurator at this point
+// Allocate a our plugin configurator
 //
-   CksConfig = new XrdCksConfig(ConfigFN, &Eroute, retc,
-                                XrdVERSIONINFOVAR(XrdOfs));
-   if (!retc) NoGo = 1;
+   ofsConfig = XrdOfsConfigPI::New(ConfigFN, &Config, &Eroute);
 
 // If there is no config file, return with the defaults sets.
 //
@@ -156,8 +166,14 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
            //
            while((var = Config.GetMyFirstWord()))
                 {if (!strncmp(var, "ofs.", 4)
-                 ||  !strcmp(var, "all.role"))
-                    if (ConfigXeq(var+4,Config,Eroute)) {Config.Echo();NoGo=1;}
+                 ||  !strcmp(var, "all.role")
+                 ||  !strcmp(var, "all.subcluster"))
+                   {if (ConfigXeq(var+4,Config,Eroute)) {Config.Echo();NoGo=1;}}
+                    else if (!strcmp(var, "oss.defaults")
+                         ||  !strcmp(var, "all.export"))
+                            {xexp(Config, Eroute, *var == 'a');
+                             Config.noEcho();
+                            }
                 }
 
            // Now check if any errors occured during file i/o
@@ -168,12 +184,9 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
            Config.Close();
           }
 
-// Determine whether we should initialize authorization
+// If no exports were specified, the default is that we are writable
 //
-   if (Options & Authorize)
-      {if (setupAuth(Eroute)) NoGo = 1;
-          else XrdOfsTPC::Init(Authorization);
-      }
+   if (ossRW == ' ') ossRW = 'w';
 
 // Check if redirection wanted
 //
@@ -185,21 +198,23 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
       {if ((j = Options & haveRole) && (i ^ j))
           {free(myRole); myRole = strdup(theRole(i));
            Eroute.Say("Config warning: command line role options override "
-                       "config file; 'ofs.role", myRole, "' in effect.");
+                       "config file; 'all.role", myRole, "' in effect.");
           }
        Options &= ~(haveRole);
        Options |= i;
       }
 
+// Export our role if we actually have one
+//
+   if (myRole) XrdOucEnv::Export("XRDROLE", myRole);
+
 // Set the redirect option for other layers
 //
    if (Options & isManager)
            XrdOucEnv::Export("XRDREDIRECT", (Options & isMeta ? "M" : "R"));
       else XrdOucEnv::Export("XRDREDIRECT", "0");
 
-// Configure the storage system at this point. This must be done prior to
-// configuring cluster processing. First check if we will be proxying.
-// If so, then substitute the proxy plug-in where we can.
+// If we are a proxy, then figure out where the prosy storge system resides
 //
    if ((Options & isProxy) && !(Options & isManager))
       {char buff[2048], *bp, *libofs = getenv("XRDOFSLIB");
@@ -208,12 +223,8 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
                 while(bp != buff && *(bp-1) != '/') bp--;
                }
        strcpy(bp, "libXrdPss.so");
-       if (OssLib) Eroute.Say("Config warning: ",
-                   "specified osslib overrides default proxy lib.");
-          else OssLib = strdup(buff);
-       if (CksConfig->Manager()) Eroute.Say("Config warning: ",
-                   "specified ckslib overrides default proxy lib.");
-          else CksConfig->Manager(buff, 0);
+       ofsConfig->Default(XrdOfsConfigPI::theOssLib, buff, 0);
+       ofsConfig->Default(XrdOfsConfigPI::theCksLib, buff, 0);
       }
 
 // Configure third party copy but only if we are not a manager or a proxy
@@ -228,18 +239,29 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
 //
    if (!(Options & isManager) && !evrObject.Init(&Eroute)) NoGo = 1;
 
-// Now configure the storage system
+// Determine whether we should load authorization
 //
-   if (!(XrdOfsOss = XrdOssGetSS(Eroute.logger(), ConfigFN, OssLib, OssParms,
-                                 XrdVERSIONINFOVAR(XrdOfs)))) NoGo = 1;
-      else XrdOfsTPC::Init(XrdOfsOss);
+   int piOpts = XrdOfsConfigPI::allXXXLib;
+   if (!(Options & Authorize)) piOpts &= ~XrdOfsConfigPI::theAutLib;
+
+// Now load all of the required plugins
+//
+   if (!ofsConfig->Load(piOpts, EnvInfo)) NoGo = 1;
+      else {ofsConfig->Plugin(XrdOfsOss);
+            ofsConfig->Plugin(Cks);
+            CksPfn = !ofsConfig->OssCks();
+            if (Options & Authorize)
+               {ofsConfig->Plugin(Authorization);
+                XrdOfsTPC::Init(Authorization);
+               }
+           }
 
 // Initialize redirection.  We type te herald here to minimize confusion
 //
    if (Options & haveRole)
       {Eroute.Say("++++++ Configuring ", myRole, " role. . .");
        if (ConfigRedir(Eroute, EnvInfo))
-          {Eroute.Emsg("Config", "Unable to create cms client object.");
+          {Eroute.Emsg("Config", "Unable to create cluster management client.");
            NoGo = 1;
           }
       }
@@ -260,21 +282,18 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
           fwdTRUNC.Reset();
          }
 
-// Configure checksums even for managers
-//
-   if (!NoGo)
-      NoGo |= (Cks = CksConfig->Configure(0, CksRdsz)) == 0;
-   delete CksConfig;
-   CksConfig = 0;
-
 // If we need to send notifications, initialize the interface
 //
    if (!NoGo && evsObject) NoGo = evsObject->Start(&Eroute);
 
 // If POSC processing is enabled (as by default) do it. Warning! This must be
 // the last item in the configuration list as we need a working filesystem.
+// Note that in proxy mode we always disable posc!
 //
-   if (poscAuto != -1 && !NoGo) NoGo |= ConfigPosc(Eroute);
+   if (getenv("XRDXROOTD_NOPOSC"))
+      {if (poscAuto != -1 && !NoGo)
+          Eroute.Say("Config POSC has been disabled by the osslib plugin.");
+      } else if (poscAuto != -1 && !NoGo) NoGo |= ConfigPosc(Eroute);
 
 // Setup statistical monitoring
 //
@@ -283,6 +302,7 @@ int XrdOfs::Configure(XrdSysError &Eroute, XrdOucEnv *EnvInfo) {
 // Display final configuration
 //
    if (!NoGo) Config_Display(Eroute);
+   delete ofsConfig; ofsConfig = 0;
 
 // All done
 //
@@ -309,29 +329,19 @@ void XrdOfs::Config_Display(XrdSysError &Eroute)
         else     pval = (poscAuto ? "auto" : "manual");
 
      snprintf(buff, sizeof(buff), "Config effective %s ofs configuration:\n"
-                                  "       ofs.role %s\n"
+                                  "       all.role %s\n"
                                   "%s"
-                                  "%s%s%s"
                                   "       ofs.maxdelay   %d\n"
-                                  "%s%s%s%s%s"
-                                  "%s%s%s"
-                                  "       ofs.persist    %s hold %d%s%s%s"
+                                  "       ofs.persist    %s hold %d%s%s\n"
                                   "       ofs.trace      %x",
               cloc, myRole,
               (Options & Authorize ? "       ofs.authorize\n" : ""),
-              (AuthLib                   ? "       ofs.authlib " : ""),
-              (AuthLib ? AuthLib : ""), (AuthLib ? "\n" : ""),
                MaxDelay,
-              (CmsLib                    ? "       ofs.cmslib " : ""),
-              (CmsLib ? CmsLib : ""), (CmsParms?" " : ""),
-              (CmsParms? CmsParms : ""), (CmsLib ? "\n" : ""),
-              (OssLib                    ? "       ofs.osslib " : ""),
-              (OssLib ? OssLib : ""), (OssLib ? "\n" : ""),
                pval, poscHold, (poscLog ? " logdir " : ""),
-               (poscLog ? poscLog    : ""), (poscLog ? "\n" : ""),
-              OfsTrace.What);
+               (poscLog ? poscLog    : ""), OfsTrace.What);
 
      Eroute.Say(buff);
+     ofsConfig->Display();
 
      if (Options & Forwarding)
         {*fwbuff = 0;
@@ -490,8 +500,7 @@ int XrdOfs::ConfigPosc(XrdSysError &Eroute)
   
 int XrdOfs::ConfigRedir(XrdSysError &Eroute, XrdOucEnv *EnvInfo)
 {
-   XrdSysPlugin *myLib;
-   XrdCmsClient *(*CmsPI)(XrdSysLogger *, int, int, XrdOss *);
+   XrdCmsClient_t CmsPI;
    XrdSysLogger *myLogger = Eroute.logger();
    int isRedir = Options & isManager;
    int RMTopts = (Options & isServer ? XrdCms::IsTarget : 0)
@@ -500,30 +509,31 @@ int XrdOfs::ConfigRedir(XrdSysError &Eroute, XrdOucEnv *EnvInfo)
    int TRGopts = (Options & isProxy  ? XrdCms::IsProxy  : 0)
                | (isRedir ? XrdCms::IsRedir : 0) | XrdCms::IsTarget;
 
-// If a cmslib was specified then create a plugin object
+// Get the cms object creator plugin
 //
-   if (CmsLib)
-      {XrdSysPlugin myLib(&Eroute,CmsLib,"cmslib",&XrdVERSIONINFOVAR(XrdOfs));
-       CmsPI = (XrdCmsClient *(*)(XrdSysLogger *, int, int, XrdOss *))
-                                  (myLib.getPlugin("XrdCmsGetClient"));
-       if (!CmsPI) return 1;
-       myLib.Persist();
-      }
+   ofsConfig->Plugin(CmsPI);
 
 // For manager roles, we simply do a standard config
 //
    if (isRedir) 
-      {     if (CmsLib) Finder = CmsPI(myLogger, RMTopts, myPort, XrdOfsOss);
+      {     if (CmsPI)  Finder = CmsPI(myLogger, RMTopts, myPort, XrdOfsOss);
        else if (XrdCmsFinderRMT::VCheck(XrdVERSIONINFOVAR(XrdOfs)))
                         Finder = (XrdCmsClient *)new XrdCmsFinderRMT(myLogger,
                                                                 RMTopts,myPort);
        else return 1;
        if (!Finder) return 1;
-       if (!Finder->Configure(ConfigFN, CmsParms, EnvInfo))
+       if (!ofsConfig->Configure(Finder, EnvInfo))
           {delete Finder; Finder = 0; return 1;}
        if (EnvInfo) EnvInfo->PutPtr("XRDCMSMANLIST", Finder->Managers());
       }
 
+// If we are a subcluster for another cluster then we can only be so if we
+// are a pure manager. If a subcluster directive was encountered and this is
+// not true we need to turn that off here. Subclusters need a target finder
+// just like supervisors eventhough we are not a supervisor.
+//
+   if ((Options & haveRole) != isManager) Options &= ~SubCluster;
+
 // For server roles find the port number and create the object. We used to pass
 // the storage system object to the finder to allow it to process cms storage
 // requests. The cms no longer sends such requests so there is no need to do
@@ -531,20 +541,21 @@ int XrdOfs::ConfigRedir(XrdSysError &Eroute, XrdOucEnv *EnvInfo)
 // finder is created. So, it's just as well we pass a numm pointer. At some
 // point the finder should remove all storage system related code.
 //
-   if (Options & (isServer | (isPeer & ~isManager)))
+   if (Options & (isServer | SubCluster | (isPeer & ~isManager)))
       {if (!myPort)
           {Eroute.Emsg("Config", "Unable to determine server's port number.");
            return 1;
           }
-            if (CmsLib) Balancer = CmsPI(myLogger, TRGopts, myPort, XrdOfsOss);
+            if (CmsPI)  Balancer = CmsPI(myLogger, TRGopts, myPort, XrdOfsOss);
        else if (XrdCmsFinderTRG::VCheck(XrdVERSIONINFOVAR(XrdOfs)))
                         Balancer = (XrdCmsClient *)new XrdCmsFinderTRG(myLogger,
                                                                 TRGopts,myPort);
        else return 1;
        if (!Balancer) return 1;
-       if (!Balancer->Configure(ConfigFN, CmsParms, EnvInfo))
+       if (!ofsConfig->Configure(Balancer, EnvInfo))
           {delete Balancer; Balancer = 0; return 1;}
-       if (Options & isProxy) Balancer = 0; // No chatting for proxies
+       if (Options & (isProxy | SubCluster))
+          Balancer = 0; // No chatting for proxies or subclusters
       }
 
 // All done
@@ -564,19 +575,24 @@ int XrdOfs::ConfigXeq(char *var, XrdOucStream &Config,
     // Now assign the appropriate global variable
     //
     TS_Bit("authorize",     Options, Authorize);
-    TS_Xeq("authlib",       xalib);
-    TS_Xeq("ckslib",        xclib);
+    TS_XPI("authlib",       theAutLib);
+    TS_XPI("ckslib",        theCksLib);
     TS_Xeq("cksrdsz",       xcrds);
-    TS_Xeq("cmslib",        xcmsl);
+    TS_XPI("cmslib",        theCmsLib);
     TS_Xeq("forward",       xforward);
     TS_Xeq("maxdelay",      xmaxd);
     TS_Xeq("notify",        xnot);
     TS_Xeq("notifymsg",     xnmsg);
-    TS_Xeq("osslib",        xolib);
+    TS_XPI("osslib",        theOssLib);
     TS_Xeq("persist",       xpers);
     TS_Xeq("role",          xrole);
     TS_Xeq("tpc",           xtpc);
     TS_Xeq("trace",         xtrace);
+    TS_XPI("xattrlib",      theAtrLib);
+
+    // Screen out the subcluster directive (we need to track that)
+    //
+    TS_Bit("subcluster",Options,SubCluster);
 
     // Get the actual value for simple directives
     //
@@ -591,108 +607,6 @@ int XrdOfs::ConfigXeq(char *var, XrdOucStream &Config,
     return 0;
 }
 
-/******************************************************************************/
-/*                                 x a l i b                                  */
-/******************************************************************************/
-  
-/* Function: xalib
-
-   Purpose:  To parse the directive: authlib <path> [<parms>]
-
-             <path>    the path of the authorization library to be used.
-             <parms>   optional parms to be passed
-
-  Output: 0 upon success or !0 upon failure.
-*/
-
-int XrdOfs::xalib(XrdOucStream &Config, XrdSysError &Eroute)
-{
-    char *val, parms[1024];
-
-// Get the path
-//
-   if (!(val = Config.GetWord()) || !val[0])
-      {Eroute.Emsg("Config", "authlib not specified"); return 1;}
-
-// Record the path
-//
-   if (AuthLib) free(AuthLib);
-   AuthLib = strdup(val);
-
-// Record any parms
-//
-   if (!Config.GetRest(parms, sizeof(parms)))
-      {Eroute.Emsg("Config", "authlib parameters too long"); return 1;}
-   if (AuthParm) free(AuthParm);
-   AuthParm = (*parms ? strdup(parms) : 0);
-   return 0;
-}
-
-/******************************************************************************/
-/*                                 x c l i b                                  */
-/******************************************************************************/
-  
-/* Function: xclib
-
-   Purpose:  To parse the directive: ckslib <digest> <path> [<parms>]
-
-             <digest>  the name of the checksum. The special name "*" is used
-                       load the checksum manager library.
-             <path>    the path of the checksum library to be used.
-             <parms>   optional parms to be passed
-
-  Output: 0 upon success or !0 upon failure.
-*/
-
-int XrdOfs::xclib(XrdOucStream &Config, XrdSysError &Eroute)
-{
-// Return the result
-//
-   return CksConfig->ParseLib(Config);
-}
-  
-/******************************************************************************/
-/*                                 x c m s l                                  */
-/******************************************************************************/
-  
-/* Function: xcmsl
-
-   Purpose:  To parse the directive: cmslib <path> [<parms>]
-
-             <path>    the path of the cms library to be used.
-             <parms>   optional parms to be passed
-
-  Output: 0 upon success or !0 upon failure.
-*/
-
-int XrdOfs::xcmsl(XrdOucStream &Config, XrdSysError &Eroute)
-{
-    char *val, parms[2048];
-    int pl;
-
-// Get the path and parms
-//
-   if (!(val = Config.GetWord()) || !val[0])
-      {Eroute.Emsg("Config", "cmslib not specified"); return 1;}
-
-// Set the CmsLib pointer
-//
-   if (CmsLib) free(CmsLib);
-   CmsLib = strdup(val);
-
-// Combine the path and parameters
-//
-   *parms = 0;
-   if (!Config.GetRest(parms, sizeof(parms)))
-      {Eroute.Emsg("Config", "cmslib parameters too long"); return 1;}
-
-// Record the parameters, if any
-//
-   if (CmsParms) free(CmsParms);
-   CmsParms = (*parms ? strdup(parms) : 0);
-   return 0;
-}
-
 /******************************************************************************/
 /*                                 x c r d s                                  */
 /******************************************************************************/
@@ -723,7 +637,52 @@ int XrdOfs::xcrds(XrdOucStream &Config, XrdSysError &Eroute)
 // Now convert it
 //
    if (XrdOuca2x::a2sz(Eroute, "cksrdsz size", val, &rdsz, 1, maxRds)) return 1;
-   CksRdsz = rdsz;
+   ofsConfig->SetCksRdSz(static_cast<int>(rdsz));
+   return 0;
+}
+  
+/******************************************************************************/
+/*                                  x e x p                                   */
+/******************************************************************************/
+  
+/* Function: xexp
+
+   Purpose:  To prescan the all.export and oss.defaults directives to determine
+             if we have any writable paths.
+
+   Output: 0 upon success or !0 upon failure.
+*/
+
+int XrdOfs::xexp(XrdOucStream &Config, XrdSysError &Eroute, bool isExport)
+{
+   static struct rwOpts {const char *opname; int isRW;} rwtab[] =
+                        {{"r/o",      0}, {"readonly",    0},
+                         {"forcero",  0}, {"notwritable", 0},
+                         {"writable", 1}, {"r/w",         1}
+                        };
+   static bool defRW = true;
+   int isrw = -1, numopts = sizeof(rwtab)/sizeof(struct rwOpts);
+   char *val;
+
+// If this is an export and we already know that we have a writable path, return
+// Otherwise, scan over the path argument.
+//
+   if (isExport && (ossRW == 'w' || !(val = Config.GetWord()))) return 0;
+
+// Throw away path and scan all the options looking for something of interest
+//
+   while((val = Config.GetWord()))
+        {for (int i = 0; i < numopts; i++)
+             if (!strcmp(val, rwtab[i].opname)) {isrw = rwtab[i].isRW; break;}
+        }
+
+// Handle result depending if this is an export or a defaults
+//
+   if (isrw < 0) isrw = defRW;
+   if (isExport) ossRW = (isrw ? 'w'  : 'r');
+      else      {defRW = (isrw ? true : false);
+                 if (ossRW == ' ' && !isrw) ossRW = 'r';
+                }
    return 0;
 }
   
@@ -1044,47 +1003,6 @@ int XrdOfs::xnot(XrdOucStream &Config, XrdSysError &Eroute)
 //
    return 0;
 }
-  
-/******************************************************************************/
-/*                                 x o l i b                                  */
-/******************************************************************************/
-  
-/* Function: xolib
-
-   Purpose:  To parse the directive: osslib <path> [<parms>]
-
-             <path>    the path of the oss library to be used.
-             <parms>   optional parms to be passed
-
-  Output: 0 upon success or !0 upon failure.
-*/
-
-int XrdOfs::xolib(XrdOucStream &Config, XrdSysError &Eroute)
-{
-    char *val, parms[2048];
-
-// Get the path and parms
-//
-   if (!(val = Config.GetWord()) || !val[0])
-      {Eroute.Emsg("Config", "osslib not specified"); return 1;}
-
-// Record the path
-//
-   if (OssLib) free(OssLib);
-   OssLib = strdup(val);
-
-// Get any parameters
-//
-   *parms = 0;
-   if (!Config.GetRest(parms, sizeof(parms)))
-      {Eroute.Emsg("Config", "osslib parameters too long"); return 1;}
-
-// Record the parameters
-//
-   if (OssParms) free(OssParms);
-   OssParms = (*parms ? strdup(parms) : 0);
-   return 0;
-}
 
 /******************************************************************************/
 /*                                 x p e r s                                  */
@@ -1158,38 +1076,28 @@ int XrdOfs::xpers(XrdOucStream &Config, XrdSysError &Eroute)
 
 /* Function: xrole
 
-   Purpose:  Parse: role { {[meta] | [peer] [proxy]} manager
-                           | peer | proxy | [proxy]  server
-                           |                [proxy]  supervisor
+   Purpose:  Parse: role { {[meta] | [proxy]} manager
+                           |         [proxy]  server
+                           |         [proxy]  supervisor
                          } [if ...]
 
              manager    xrootd: act as a manager (redirecting server). Prefixes:
                                 meta  - connect only to manager meta's
-                                peer  - ignored
                                 proxy - ignored
                         cmsd:   accept server subscribes and redirectors. Prefix
                                 modifiers do the following:
                                 meta  - No other managers apply
-                                peer  - subscribe to other managers as a peer
                                 proxy - manage a cluster of proxy servers
 
-             peer       xrootd: same as "peer manager"
-                        olbd:   same as "peer manager" but no server subscribers
-                                are required to function (i.e., run stand-alone).
-
-             proxy      xrootd: act as a server but supply data from another 
-                                server. No local olbd is present or required.
-                        olbd:   Generates an error as this makes no sense.
-
              server     xrootd: act as a server (supply local data). Prefix
                                 modifications do the following:
                                 proxy - server is part of a cluster. A local
-                                        olbd is required.
-                        olbd:   subscribe to a manager, possibly as a proxy.
+                                        cmsd is required.
+                        cmsd:   subscribe to a manager, possibly as a proxy.
 
              supervisor xrootd: equivalent to manager. The prefix modification
                                 is ignored.
-                        olbd:   equivalent to manager but also subscribe to a
+                        cmsd:   equivalent to manager but also subscribe to a
                                 manager. When proxy is specified, then subscribe
                                 as a proxy and only accept proxies.
 
@@ -1206,68 +1114,64 @@ int XrdOfs::xpers(XrdOucStream &Config, XrdSysError &Eroute)
 int XrdOfs::xrole(XrdOucStream &Config, XrdSysError &Eroute)
 {
    const int resetit = ~haveRole;
-   char role[64];
-   char *val;
-   int rc, mopt = 0, qopt = 0, ropt = 0, sopt = 0;
+    XrdCmsRole::RoleID roleID;
+    char *val, *Tok1, *Tok2;
+    int rc, ropt = 0;
 
-   *role = '\0';
-   if (!(val = Config.GetWord()))
+// Get the first token
+//
+   if (!(val = Config.GetWord()) || !strcmp(val, "if"))
       {Eroute.Emsg("Config", "role not specified"); return 1;}
+   Tok1 = strdup(val);
 
-
-// Scan for "meta" o/w "peer" or "proxy"
+// Get second token which might be an "if"
 //
-   if (!strcmp("meta", val))
-      {mopt = isMeta; strcpy(role, val); val = Config.GetWord();}
-      else {if (!strcmp("peer", val))
-               {qopt = isPeer; strcpy(role, val);
-                val = Config.GetWord();
-               }
-            if (val && !strcmp("proxy", val))
-               {ropt = isProxy;
-                if (qopt) strcat(role, " ");
-                strcat(role, val);
-                val = Config.GetWord();
-               }
-           }
-
-// Scan for other possible alternatives
-//
-   if (val && strcmp("if", val))
-      {     if (!strcmp("manager",    val)) sopt = isManager;
-       else if (!strcmp("server",     val)) sopt = isServer;
-       else if (!strcmp("supervisor", val)) sopt = isSuper;
-       else    {Eroute.Emsg("Config", "invalid role -", val); return 1;}
-
-       if (mopt || qopt || ropt) strcat(role, " ");
-       strcat(role, val);
+   if ((val = Config.GetWord()) && strcmp(val, "if"))
+      {Tok2 = strdup(val);
        val = Config.GetWord();
-      }
+      } else Tok2 = 0;
 
-// Scan for invalid roles: peer proxy | peer server | {peer} supervisor
+// Process the if at this point
 //
-   if (((mopt || (qopt && ropt)) && !sopt)
-   ||  ((mopt || qopt) && sopt == isServer)
-   ||  ((mopt || qopt) && sopt == isSuper))
-      {Eroute.Emsg("Config", "invalid role -", role); return 1;}
-
-// Make sure a role was specified
-//
-    if (!(ropt = mopt | qopt | ropt | sopt))
-       {Eroute.Emsg("Config", "role not specified"); return 1;}
-
-// Pick up optional "if"
-//
-    if (val && !strcmp("if", val))
-       if ((rc = XrdOucUtils::doIf(&Eroute,Config,"role directive",
+   if (val && !strcmp("if", val))
+      {if ((rc = XrdOucUtils::doIf(&Eroute,Config,"role directive",
                                    getenv("XRDHOST"), XrdOucUtils::InstName(1),
                                    getenv("XRDPROG"))) <= 0)
-          {if (!rc) Config.noEcho(); return (rc < 0);}
+          {free(Tok1); if (Tok2) free(Tok2);
+           if (!rc) Config.noEcho();
+           return (rc < 0);
+          }
+      }
+
+// Convert the role names to a role ID, if possible
+//
+   roleID = XrdCmsRole::Convert(Tok1, Tok2);
+
+// Set markers based on the role we have
+//
+   rc = 0;
+   switch(roleID)
+         {case XrdCmsRole::MetaManager:  ropt = isManager | isMeta ; break;
+          case XrdCmsRole::Manager:      ropt = isManager          ; break;
+          case XrdCmsRole::Supervisor:   ropt = isSuper            ; break;
+          case XrdCmsRole::Server:       ropt = isServer           ; break;
+          case XrdCmsRole::ProxyManager: ropt = isManager | isProxy; break;
+          case XrdCmsRole::ProxySuper:   ropt = isSuper   | isProxy; break;
+          case XrdCmsRole::ProxyServer:  ropt = isServer  | isProxy; break;
+          default: Eroute.Emsg("Config", "invalid role -", Tok1, Tok2); rc = 1;
+         }
+
+// Release storage and return if an error occured
+//
+   free(Tok1);
+   if (Tok2) free(Tok2);
+   if (rc) return rc;
 
 // Set values
 //
     free(myRole);
-    myRole = strdup(role);
+    myRole = strdup(XrdCmsRole::Name(roleID));
+    strcpy(myRType, XrdCmsRole::Type(roleID));
     Options &= resetit;
     Options |= ropt;
     return 0;
@@ -1283,7 +1187,8 @@ int XrdOfs::xrole(XrdOucStream &Config, XrdSysError &Eroute)
                                          [logok] [xfr <n>] [allow <parms>]
                                          [require {all|client|dest} <auth>[+]]
                                          [restrict <path>] [streams <num>]
-                                         [pgm <path> [parms]]
+                                         [echo] [scan {stderr | stdout}]
+                                         [autorm] [pgm <path> [parms]]
 
              parms: [dn <name>] [group <grp>] [host <hn>] [vo <vo>]
 
@@ -1301,6 +1206,10 @@ int XrdOfs::xrole(XrdOucStream &Config, XrdSysError &Eroute)
                      valid authentication mechanisms. If the <auth> is suffixed
                      by a plus, then the request must also be encrypted using
                      the authentication's session key.
+             echo    echo the pgm's output to the log.
+             autorm  Remove file when copy fails.
+             scan    scan fr error messages either in stderr or stdout. The
+                     default is to scan both.
              pgm     specifies the transfer command with optional paramaters.
                      It must be the last parameter on the line.
 
@@ -1310,7 +1219,7 @@ int XrdOfs::xrole(XrdOucStream &Config, XrdSysError &Eroute)
 int XrdOfs::xtpc(XrdOucStream &Config, XrdSysError &Eroute)
 {
    XrdOfsTPC::iParm Parms;
-   char *vcksum = 0, *val, pgm[1024];
+   char *val, pgm[1024];
    int  reqType;
    *pgm = 0;
 
@@ -1326,7 +1235,17 @@ int XrdOfs::xtpc(XrdOucStream &Config, XrdSysError &Eroute)
              Parms.Ckst = strdup(val);
              continue;
             }
+         if (!strcmp(val, "scan"))
+            {if (!(val = Config.GetWord()))
+                {Eroute.Emsg("Config","scan type not specified"); return 1;}
+                  if (strcmp(val, "stderr")) Parms.Grab = -2;
+             else if (strcmp(val, "stdout")) Parms.Grab = -1;
+                {Eroute.Emsg("Config","invalid scan type -",val); return 1;}
+             continue;
+            }
+         if (!strcmp(val, "echo"))  {Parms.xEcho = 1; continue;}
          if (!strcmp(val, "logok")) {Parms.Logok = 1; continue;}
+         if (!strcmp(val, "autorm")){Parms.autoRM = 1; continue;}
          if (!strcmp(val, "pgm"))
             {if (!Config.GetRest(pgm, sizeof(pgm)))
                 {Eroute.Emsg("Config", "tpc command line too long"); return 1;}
@@ -1490,39 +1409,6 @@ int XrdOfs::xtrace(XrdOucStream &Config, XrdSysError &Eroute)
 //
    return 0;
 }
-
-/******************************************************************************/
-/*                             s e t u p A u t h                              */
-/******************************************************************************/
-
-int XrdOfs::setupAuth(XrdSysError &Eroute)
-{
-   extern XrdAccAuthorize *XrdAccDefaultAuthorizeObject
-                          (XrdSysLogger   *lp,    const char   *cfn,
-                           const char     *parm,  XrdVersionInfo &vInfo);
-
-   XrdSysPlugin    *myLib;
-   XrdAccAuthorize *(*ep)(XrdSysLogger *, const char *, const char *);
-
-// Authorization comes from the library or we use the default
-//
-   if (!AuthLib) return 0 == (Authorization = XrdAccDefaultAuthorizeObject
-                             (Eroute.logger(), ConfigFN, AuthParm,
-                              XrdVERSIONINFOVAR(XrdOfs)));
-
-// Create a plugin object
-//
-  {XrdSysPlugin myLib(&Eroute, AuthLib, "authlib", &XrdVERSIONINFOVAR(XrdOfs));
-   ep = (XrdAccAuthorize *(*)(XrdSysLogger *, const char *, const char *))
-                             (myLib.getPlugin("XrdAccAuthorizeObject"));
-   if (!ep) return 1;
-   myLib.Persist();
-  }
-
-// Get the Object now
-//
-   return 0 == (Authorization = ep(Eroute.logger(), ConfigFN, AuthParm));
-}
   
 /******************************************************************************/
 /*                               t h e R o l e                                */
