@@ -102,38 +102,6 @@ int XrdPosixMap::mapCode(int rc)
    return ENOMSG;
 }
 
-/******************************************************************************/
-/* Private:                     m a p E r r o r                               */
-/******************************************************************************/
-  
-int XrdPosixMap::mapError(int rc)
-{
-    switch(rc)
-       {case kXR_ArgInvalid:    return EINVAL;
-        case kXR_ArgMissing:    return EINVAL;
-        case kXR_ArgTooLong:    return ENAMETOOLONG;
-        case kXR_FileLocked:    return EDEADLK;
-        case kXR_FileNotOpen:   return EBADF;
-        case kXR_FSError:       return EIO;
-        case kXR_InvalidRequest:return EEXIST;
-        case kXR_IOError:       return EIO;
-        case kXR_NoMemory:      return ENOMEM;
-        case kXR_NoSpace:       return ENOSPC;
-        case kXR_NotAuthorized: return EACCES;
-        case kXR_NotFound:      return ENOENT;
-        case kXR_ServerError:   return ENOMSG;
-        case kXR_Unsupported:   return ENOSYS;
-        case kXR_noserver:      return EHOSTUNREACH;
-        case kXR_NotFile:       return ENOTBLK;
-        case kXR_isDirectory:   return EISDIR;
-        case kXR_Cancelled:     return ECANCELED;
-        case kXR_ChkLenErr:     return EDOM;
-        case kXR_ChkSumErr:     return EDOM;
-        case kXR_inProgress:    return EINPROGRESS;
-        default:                return ENOMSG;
-       }
-}
-
 /******************************************************************************/
 /*                           M o d e 2 A c c e s s                            */
 /******************************************************************************/
@@ -171,7 +139,7 @@ int XrdPosixMap::Result(const XrdCl::XRootDStatus &Status)
 //
    if (Status.code == XrdCl::errErrorResponse)
       {eText = Status.GetErrorMessage();
-       eNum  = mapError(Status.errNo);
+       eNum  = XProtocol::toErrno(Status.errNo);
       } else {
        eText = Status.ToStr();
        eNum  = (Status.errNo ? Status.errNo : mapCode(Status.code));
