@@ -57,7 +57,11 @@ XrdXrootdXPath        XrdXrootdProtocol::RPList;
 XrdXrootdXPath        XrdXrootdProtocol::RQList;
 XrdXrootdXPath        XrdXrootdProtocol::XPList;
 XrdSfsFileSystem     *XrdXrootdProtocol::osFS;
-char                 *XrdXrootdProtocol::FSLib    = 0;
+XrdSfsFileSystem     *XrdXrootdProtocol::digFS    = 0;
+char                 *XrdXrootdProtocol::FSLib[2] = {0, 0};
+int                   XrdXrootdProtocol::FSLvn[2] = {0, 0};
+char                 *XrdXrootdProtocol::digLib   = 0;
+char                 *XrdXrootdProtocol::digParm  = 0;
 XrdXrootdFileLock    *XrdXrootdProtocol::Locker;
 XrdSecService        *XrdXrootdProtocol::CIA      = 0;
 char                 *XrdXrootdProtocol::SecLib   = 0;
@@ -68,8 +72,12 @@ XrdSysError           XrdXrootdProtocol::eDest(0, "Xrootd");
 XrdXrootdStats       *XrdXrootdProtocol::SI;
 XrdXrootdJob         *XrdXrootdProtocol::JobCKS   = 0;
 char                 *XrdXrootdProtocol::JobCKT   = 0;
+XrdOucTList          *XrdXrootdProtocol::JobCKTLST= 0;
+XrdOucReqID          *XrdXrootdProtocol::PrepID   = 0;
 
 char                 *XrdXrootdProtocol::Notify = 0;
+const char           *XrdXrootdProtocol::myCName= 0;
+int                   XrdXrootdProtocol::myCNlen= 0;
 int                   XrdXrootdProtocol::hailWait;
 int                   XrdXrootdProtocol::readWait;
 int                   XrdXrootdProtocol::Port;
@@ -77,8 +85,8 @@ int                   XrdXrootdProtocol::Window;
 int                   XrdXrootdProtocol::WANPort;
 int                   XrdXrootdProtocol::WANWindow;
 char                  XrdXrootdProtocol::isRedir = 0;
-char                  XrdXrootdProtocol::chkfsV  = 0;
 char                  XrdXrootdProtocol::JobLCL  = 0;
+char                  XrdXrootdProtocol::JobCKCGI=0;
 XrdNetSocket         *XrdXrootdProtocol::AdminSock= 0;
 
 int                   XrdXrootdProtocol::hcMax        = 28657; // const for now
@@ -108,7 +116,7 @@ int                   XrdXrootdProtocol::myPID = static_cast<int>(getpid());
 int                   XrdXrootdProtocol::myRole = 0;
 int                   XrdXrootdProtocol::myRolf = 0;
 
-struct XrdXrootdProtocol::RD_Table XrdXrootdProtocol::Route[RD_Num] = {{0,0}};
+struct XrdXrootdProtocol::RD_Table XrdXrootdProtocol::Route[RD_Num];
 
 /******************************************************************************/
 /*            P r o t o c o l   M a n a g e m e n t   S t a c k s             */
@@ -235,13 +243,13 @@ static  const int hpSZ = hsSZ + prSZ;
 static  struct hs_response
                {kXR_unt16 streamid;
                 kXR_unt16 status;
-                kXR_unt32 rlen;
-                kXR_unt32 pval;
-                kXR_unt32 styp;
+                kXR_unt32 rlen;   // Specified as kXR_int32 in doc!
+                kXR_unt32 pval;   // Specified as kXR_int32 in doc!
+                kXR_unt32 styp;   // Specified as kXR_int32 in doc!
                } hsresp={0, 0, htonl(8), // isRedir == 'M' -> MetaManager
                          htonl(kXR_PROTOCOLVERSION),
-                         (isRedir ? htonl(kXR_LBalServer)
-                                  : htonl(kXR_DataServer))};
+                         (isRedir ? htonl((unsigned int)kXR_LBalServer)
+                                  : htonl((unsigned int)kXR_DataServer))};
 
 XrdXrootdProtocol *xp;
 int dlen, rc;
@@ -312,6 +320,7 @@ int dlen, rc;
    xp->Response.Set(lp);
    strcpy(xp->Entity.prot, "host");
    xp->Entity.host = (char *)lp->Host();
+   xp->Entity.addrInfo = lp->AddrInfo();
    return (XrdProtocol *)xp;
 }
  
@@ -554,7 +563,7 @@ void XrdXrootdProtocol::Recycle(XrdLink *lp, int csec, const char *reason)
 
 // Push ourselves on the stack
 //
-   ProtStack.Push(&ProtLink);
+   if (Response.isOurs()) ProtStack.Push(&ProtLink);
 }
 
 /******************************************************************************/
@@ -597,16 +606,16 @@ int XrdXrootdProtocol::CheckSum(XrdOucStream *Stream, char **argv, int argc)
    XrdOucErrInfo myInfo("CheckSum");
    int rc, ecode;
 
-// The arguments must have <name> <path> (i.e. argc >= 2)
+// The arguments must have <name> <cstype> <path> (i.e. argc >= 3)
 //
-   if (argc < 2)
+   if (argc < 3)
       {Stream->PutLine("Internal error; not enough checksum args!");
        return 8;
       }
 
 // Issue the checksum calculation (that's all we do here).
 //
-   rc = osFS->chksum(XrdSfsFileSystem::csCalc, JobCKT, argv[1], myInfo);
+   rc = osFS->chksum(XrdSfsFileSystem::csCalc, argv[1], argv[2], myInfo);
 
 // Return result regardless of what it is
 //
@@ -626,10 +635,18 @@ void XrdXrootdProtocol::Cleanup()
    XrdXrootdPio *pioP;
    int i;
 
+// Release any internal monitoring information
+//
+   if (Entity.moninfo) {free(Entity.moninfo); Entity.moninfo = 0;}
+
 // If we have a buffer, release it
 //
    if (argp) {BPool->Release(argp); argp = 0;}
 
+// Notify the filesystem of a disconnect prior to deleting file tables
+//
+   if (Status != XRD_BOUNDPATH) osFS->Disc(Client);
+
 // Delete the FTab if we have it
 //
    if (FTab)
@@ -737,6 +754,7 @@ void XrdXrootdProtocol::Reset()
    rvSeq              = 0;
    pioFree = pioFirst = pioLast = 0;
    isActive = isDead  = isNOP = isBound = 0;
+   rdType             = 0;
    memset(&Entity, 0, sizeof(Entity));
    memset(Stream,  0, sizeof(Stream));
 }
